/**
    @file

    @brief
        This file describes the error codes used in the aDIOLib API.

    @note
        This file is a modified version of the header file output by 'mc.exe'
        when run on aDIOLib_Error.mc. If aDIOLib_Error.mc is changed, 'mc.exe'
        must be run manually and the resulting header file must be merged with
        this file.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Error.h 54547 2011-07-15 20:46:51Z jstrouse $
*/

#ifndef __ADIOLIB_ERROR_H__
#define __ADIOLIB_ERROR_H__

#ifdef __cplusplus
extern "C" {
#endif


//============================================================================
//! @defgroup aDIO_API aDIOLib API
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Error aDIOLib Error Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Error_Def aDIOLib Error Defines
//! @{
//============================================================================

/**
    Creates a specific type for error codes of the aDIOLib.

    For a list of valid values, see the ADIO_ERROR_CODE_* values in
    aDIOLib_Error.h
*/
typedef uint32 aDIO_Error;

/**
    Creates a specific type for error categories of the aDIOLib.

    For a list of valid values, see the ADIO_ERROR_CATEGORY_* values in
    aDIOLib_Error.h
*/
typedef WORD aDIO_Error_Cat;

/**
    Defines the maximum size, in bytes, of an error message after all
    substitutions have been made.
*/
#define ADIO_ERROR_MAX_MSG_LENGTH       2048

/**
    Defines the maximum size, in bytes, of an error format string.
*/
#define ADIO_ERROR_MAX_FORMAT_LENGTH    80

/**
    Defines the number of error codes used in the aDIOLib.
*/
#define ADIO_ERROR_COUNT                11

/**
    Defines the number of error categories used in the aDIOLib.
*/
#define ADIO_ERROR_CATEGORY_COUNT       5

//============================================================================
//! @} End of Error Defines
//============================================================================

//============================================================================
//! @defgroup aDIO_Error_Severities aDIOLib Error Severities
//! @{
//============================================================================

/**
    Defines a severity of 'success', used to log messages that report that an
    operation completed successfully.
*/
#define ADIO_ERROR_SEVERITY_SUCCESS         0x0

/**
    Defines a severity of 'informational', used to log messages that only convey
    information.
*/
#define ADIO_ERROR_SEVERITY_INFORMATIONAL   0x1

/**
    Defines a severity of 'warning', used to log messages for non-fatal or
    potential errors.
*/
#define ADIO_ERROR_SEVERITY_WARNING         0x2

/**
    Defines a severity of 'error', used to log messages when an error occurs.
*/
#define ADIO_ERROR_SEVERITY_ERROR           0x3

//============================================================================
//! @} End of Error Severities
//============================================================================

//============================================================================
//! @defgroup aDIO_Error_Categories aDIOLib Error Categories
//! @{
//============================================================================

/**
    Defines a category value representing the aDIOLib General Module.
*/
#define ADIO_ERROR_CATEGORY_GENERAL             ((aDIO_Error_Cat)0x00000001L)

/**
    Defines a category value representing the aDIOLib General Internal Module.
*/
#define ADIO_ERROR_CATEGORY_GENERAL_INTERNAL    ((aDIO_Error_Cat)0x00000002L)

/**
    Defines a category value representing the aDIOLib Interrupt Module.
*/
#define ADIO_ERROR_CATEGORY_INTERRUPT           ((aDIO_Error_Cat)0x00000003L)

/**
    Defines a category value representing the aDIOLib Interrupt Internal Module.
*/
#define ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL  ((aDIO_Error_Cat)0x00000004L)

/**
    Defines a category value representing the aDIOLib Port Module.
*/
#define ADIO_ERROR_CATEGORY_PORT                ((aDIO_Error_Cat)0x00000005L)

//============================================================================
//! @} End of Error Categories
//============================================================================

//============================================================================
//! @defgroup aDIO_Error_Codes aDIOLib Error Codes
//! @{
//============================================================================

/**
    Defines an error code value representing a successful operation.
*/
#define ADIO_ERROR_CODE_SUCCESS          ((aDIO_Error)0x00000100L)

/**
    Defines an error code value representing an informational log message.
*/
#define ADIO_ERROR_CODE_INFO             ((aDIO_Error)0x40000200L)

/**
    Defines an error code value representing a warning (or non-fatal error).
*/
#define ADIO_ERROR_CODE_WARNING          ((aDIO_Error)0x80000300L)

/**
    Defines an error code value representing an unexpected error encountered
    during operation.
*/
#define ADIO_ERROR_CODE_UNEXPECTED       ((aDIO_Error)0xC0000400L)

/**
    Defines an error code value representing a bad parameter passed into the
    aDIOLib API.
*/
#define ADIO_ERROR_CODE_BAD_PARAM        ((aDIO_Error)0xC0000500L)

/**
    Defines an error code value representing an attempt to use functionality
    that has not yet been configured or is configured incorrectly.
*/
#define ADIO_ERROR_CODE_NOT_CONFIG       ((aDIO_Error)0xC0000600L)

/**
    Defines an error code value representing an attempt to use functionality
    that has not yet been enabled.
*/
#define ADIO_ERROR_CODE_NOT_ENABLE       ((aDIO_Error)0xC0000700L)

/**
    Defines an error code value representing an attempt to use or change
    functionality that is still in use or otherwise 'busy'.
*/
#define ADIO_ERROR_CODE_IN_USE           ((aDIO_Error)0xC0000800L)

/**
    Defines an error code value representing a failure to allocate memory.
*/
#define ADIO_ERROR_CODE_NO_MEMORY        ((aDIO_Error)0xC0000900L)

/**
    Defines an error code value representing a failed call to a Windows API
    function.
*/
#define ADIO_ERROR_CODE_WIN_API          ((aDIO_Error)0xC0000A00L)

/**
    Defines an error code value representing a failed IOCTL call to the driver.
*/
#define ADIO_ERROR_CODE_IOCTL_FAIL       ((aDIO_Error)0xC0000B00L)

//============================================================================
//! @} End of Error Codes
//============================================================================

//============================================================================
//! @defgroup aDIO_Error_Messages aDIOLib Error Messages
//! @{
//============================================================================

/**
    Defines a format string for ADIO_ERROR_CODE_SUCCESS error messages.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_SUCCESS "No error. Operation was successful."

/**
    Defines a format string for ADIO_ERROR_CODE_INFO error messages.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_INFO "Info: %s"

/**
    Defines a format string for ADIO_ERROR_CODE_WARNING error messages.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_WARNING "Warning: %s"

/**
    Defines a format string for ADIO_ERROR_CODE_UNEXPECTED error messages.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_UNEXPECTED "Unexpected Error: %s"

/**
    Defines a format string for ADIO_ERROR_CODE_BAD_PARAM error messages.

    @note
        This format string is for bad (NULL) pointer parameters.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_BAD_PARAM_PTR \
                            "Function %s, parameter %s had bad value of NULL."

/**
    Defines a format string for ADIO_ERROR_CODE_BAD_PARAM error messages.

    @note
        This format string is for bad integer parameters.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_BAD_PARAM_INT \
                            "Function %s, parameter %s had bad value of %d."

/**
    Defines a format string for ADIO_ERROR_CODE_BAD_PARAM error messages.

    @note
        This format string is for bad float parameters.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_BAD_PARAM_FLOAT \
                            "Function %s, parameter %s had bad value of %f"

/**
    Defines a format string for ADIO_ERROR_CODE_BAD_PARAM error messages.

    @note
        This format string is for bad string parameters.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_BAD_PARAM_STR \
                            "Function %s, parameter %s had bad value of \"%s\"."

/**
    Defines a format string for ADIO_ERROR_CODE_NOT_CONFIG error messages.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_NOT_CONFIG "%s must be configured before being used."

/**
    Defines a format string for ADIO_ERROR_CODE_NOT_ENABLE error messages.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_NOT_ENABLE "%s must be enabled first."

/**
    Defines a format string for ADIO_ERROR_CODE_IN_USE error messages.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_IN_USE "%s is still in use."

/**
    Defines a format string for ADIO_ERROR_CODE_NO_MEMORY error messages.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_NO_MEMORY "Memory low. Allocation of 0x%X bytes failed."

/**
    Defines a format string for ADIO_ERROR_CODE_WIN_API error messages.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_WIN_API \
            "Calling WinAPI function %s failed with GetLastError returning %u."

/**
    Defines a format string for ADIO_ERROR_CODE_IOCTL_FAIL error messages.

    @note
        This string can be at most ADIO_ERROR_MAX_FORMAT_LENGTH characters
        long, including terminating null.
*/
#define ADIO_ERROR_MSG_IOCTL_FAIL \
                    "Calling IOCTL %s failed with GetLastError returning %u."

//============================================================================
//! @} End of Error Messages
//============================================================================

//============================================================================
//! @} End of Error Module
//============================================================================

//============================================================================
//! @} End of API
//============================================================================


#ifdef __cplusplus
}
#endif

#endif // __ADIOLIB_ERROR_H__
