/**
    @file

    @brief
        This file defines the 'General' module of the aDIOLib API.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_General.h 55027 2011-08-15 20:24:09Z jstrouse $
*/

#ifndef __ADIOLIB_GENERAL_H__
#define __ADIOLIB_GENERAL_H__

#include <windows.h>
#include <stdio.h>
#include "RTD_Types.h"
#include "aDIO_Interface.h"
#include "aDIOLib_Error.h"

#ifdef __cplusplus
extern "C" {
#endif

/*
    The following ifdef block is the standard way of creating macros which make
    exporting from a DLL simpler. All files within this DLL are compiled with
    the ADIOLIB_EXPORT symbol defined on the command line. This symbol should
    not be defined on any project that uses this DLL. This way, any other
    project whose source files include this file see ADIOLIB_API functions as
    being imported from a DLL, whereas this DLL sees symbols defined with this
    macro as being exported.
*/
/** For function import/export. See source.*/
#ifdef ADIOLIB_EXPORT
    #define ADIOLIB_API __declspec(dllexport)
#else
    #define ADIOLIB_API __declspec(dllimport)
#endif


//============================================================================
//! @defgroup aDIO_API aDIOLib API
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_General aDIOLib General Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_General_Struct aDIOLib General Structures
//! @{
//============================================================================

/**
    Contains enumeration information for one aDIO device.
*/
typedef struct
{
    /**
        Contains the device number used to open this device.
    */
    uint32 dev_num;

    /**
        Contains the current status/availability of the device.
    */
    aDIO_Intrfc_Avail availability;

    /**
        Contains the base address of this aDIO's register set.
    */
    uint32 base_address;

    /**
        Contains the IRQ(interrupt) this aDIO device is using. An IRQ of zero
        means that the device has not been assigned an interrupt.
    */
    uint32 irq;

    /**
        Marks whether or not this aDIO supports Wake-on-aDIO.
    */
    BOOL has_wake;

} aDIO_Enum_Info;

/**
    Contains aDIO information used during device operation.
*/
typedef struct
{
    /**
        Holds a critical section object used to control access to all library
        state and functionality.
    */
    CRITICAL_SECTION global_lock;

    /**
        Holds a handle to the aDIO device file. (Used to send IOCTLs to the
        driver.)
    */
    HANDLE dev_file;

    /**
        Holds device/board information retrieved from the driver.
    */
    aDIO_Intrfc_Board_Info dev_info;

    /**
        Holds a critical section object used to control access to the error
        logging function and the last error message.
    */
    CRITICAL_SECTION log_section;

    /**
        Holds the last un-retrieved error message.
    */
    char last_error_msg[ADIO_ERROR_MAX_MSG_LENGTH];

    /**
        Holds a pointer to the structure holding the state of the Interrupt
        module.
    */
    void* int_state;

} aDIO_Descriptor, *aDIO_Handle;

//============================================================================
//! @} End of General Structures
//============================================================================

//============================================================================
//! @defgroup aDIO_General_Func aDIOLib General Functions
//! @{
//============================================================================

/**
    @brief
        Enumerates the aDIO devices currently installed in the system.

    @param
        DeviceList

        A pointer to an array of aDIO_Enum_Info structures. This array will be
        filled (starting at index 0) with each aDIO device's enumeration
        information.

    @param
        ListSize

        The size of the array pointed to by DeviceList. This is also the maximum
        number of devices to be enumerated.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_General_Enum(aDIO_Enum_Info* DeviceList,
                                    uint8           ListSize);

/**
    @brief
        Opens an aDIO device and readies it for use.

    @param
        DeviceInfo

        An aDIO_Enum_Info structure filled with the enumeration information of
        the aDIO device to be opened. (See aDIO_General_Enum())

    @param
        Handle

        A pointer to an aDIO_Handle variable which is filled with the handle to
        the aDIO.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_General_Open(aDIO_Enum_Info  DeviceInfo,
                                    aDIO_Handle*    Handle);

/**
    @brief
        Closes an aDIO device and performs needed cleanup.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_General_Close(aDIO_Handle Handle);

/**
    @brief
        Resets the aDIO circuitry to a 'default' state. (All registers are made
        zero.)

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_General_Reset(aDIO_Handle Handle);

/**
    @brief
        Retrieves the error message recorded for an error returned from the
        aDIOLib API.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        MessageBuffer

        A pointer to a character buffer which will be filled with the error
        message. This buffer must be at least ADIO_ERROR_MAX_MSG_LENGTH
        characters in size.

    @param
        BufferSize

        The size, in bytes, of the buffer pointed to by 'MessageBuffer'.

    @note
        If an aDIOLib_Error value other than ADIO_ERROR_CODE_SUCCESS is returned
        from a function, make sure to use this function or the messages of
        subsequent errors will be lost.

    @note
        This function is synchronized internally and may be safely called from
        multiple threads.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_General_GetLastErrorMsg( aDIO_Handle Handle,
                                                char*       MessageBuffer,
                                                uint32      BufferSize);

//============================================================================
//! @} End of General Functions
//============================================================================

//============================================================================
//! @} End of General Module
//============================================================================

//============================================================================
//! @} End of API
//============================================================================


#ifdef __cplusplus
}
#endif

#endif // __ADIOLIB_GENERAL_H__
