/**
    @file

    @brief
        This file defines the 'Interrupt' module of the aDIOLib API.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Interrupt.h 55027 2011-08-15 20:24:09Z jstrouse $
*/

#ifndef __ADIOLIB_INTERRUPT_H__
#define __ADIOLIB_INTERRUPT_H__

#include "aDIOLib_General.h"

#ifdef __cplusplus
extern "C" {
#endif


//============================================================================
//! @defgroup aDIO_API aDIOLib API
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt aDIOLib Interrupt Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt_Enum aDIOLib Interrupt Enumerations
//! @{
//============================================================================

/**
    Enumerates the types of interrupts.

    @note
        Only one interrupt type can be enabled at a time. If an interrupt type
        is enabled, all other types will become disabled.
*/
typedef enum
{
    /**
        Represents the 'Strobe' interrupt type.
    */
    ADIO_INTERRUPT_INT_TYPE_STROBE = 0,

    /**
        Represents the 'Event' interrupt type.
    */
    ADIO_INTERRUPT_INT_TYPE_EVENT,

    /**
        Represents the 'Match' interrupt type.
    */
    ADIO_INTERRUPT_INT_TYPE_MATCH,

    /**
        Represents the number of different interrupt types.

        @warning
            This is NOT a real interrupt type.
    */
    ADIO_INTERRUPT_INT_TYPE_COUNT,

} aDIO_Interrupt_Int_Type;

//============================================================================
//! @} End of Interrupt Enumerations
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt_Struct aDIOLib Interrupt Structures
//! @{
//============================================================================

/**
    Holds information that is provided to a callback function when an interrupt
    occurs.
*/
typedef struct
{
    /**
        This is the type of interrupt that occurred.
    */
    aDIO_Interrupt_Int_Type type;

    /**
        This is the reason for calling the callback function. If this value is
        not ADIO_ERROR_CODE_SUCCESS, this value describes an error that occurred
        inside the callback thread while running. If this happens, the callback
        must be removed using aDIO_Interrupt_RemoveCallback. The callback can
        then be installed again.
    */
    aDIO_Error result;

    /**
        This is the number of times this specific interrupt occurred since the
        last time the callback function was called.
    */
    uint32 interrupt_count;

    /**
        This is the value of the 'Compare' register when the interrupt occurred.

        @note
            If this callback is for more than one interrupt (interrupt_count is
            greater than 1), then this value is from the most recent interrupt
            counted.

        @note
            When a strobe or event interrupt occurs, Port 0 is latched into the
            'Compare' register.
    */
    uint8 compare_val;

    /**
        This is the value of Port 0 when the interrupt occurred.

        @note
            If this callback is for more than one interrupt (interrupt_count is
            greater than 1), then this value is from the most recent interrupt
            counted.

        @note
            This value is not latched, it is simply read during interrupt
            processing. This means there is a short interval in which the value
            of Port 0 could change from its value at the time of the interrupt.
    */
    uint8 port0_val;

    /**
        This is the value of Port 1 when the interrupt occurred.

        @note
            If this callback is for more than one interrupt (interrupt_count is
            greater than 1), then this value is from the most recent interrupt
            counted.

        @note
            This value is not latched, it is simply read during interrupt
            processing. This means there is a short interval in which the value
            of Port 1 could change from its value at the time of the interrupt.
    */
    uint8 port1_val;

    /**
        This is the user-provided pointer passed into
        aDIO_Interrupt_InstallCallback.
    */
    void* user_pointer;

} aDIO_Interrupt_Callback_Info;

//============================================================================
//! @} End of Interrupt Structures
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt_Callback aDIOLib Interrupt Callbacks
//! @{
//============================================================================

/**
    Defines the format for interrupt callback routines.

    Callbacks should be declared as:
    @code
    void MyCallback(aDIO_Interrupt_Callback_Info Info);
    @endcode

    @warning
        The given callback function will be called in its own thread context.
        Appropriate synchronization of code and data is required between the
        main thread and any callback functions.

    @note
        If an error should occur while the callback thread is running, the
        user's callback will be called before the thread exits. The 'result'
        value in the aDIO_Interrupt_Callback_Info structure will contain the
        appropriate error code.

*/
typedef void (*aDIO_Interrupt_Callback)(aDIO_Interrupt_Callback_Info Info);

//============================================================================
//! @} End of Interrupt Callbacks
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt_Func aDIOLib Interrupt Functions
//! @{
//============================================================================

/**
    @brief
        Configures interrupts and Wake-on-aDIO.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        IntEnable

        If this value is TRUE, the interrupt type given by Type will be enabled
        and all other interrupt types will be disabled. If this value is FALSE,
        all interrupt types will be disabled.

    @param
        Type

        The interrupt type to enable when enabling interrupts. If IntEnable is
        FALSE, this value is ignored.

    @param
        WakeEnable

        If TRUE, Wake-on-aDIO will be enabled, otherwise it will be disabled.
        If the aDIO device does not support Wake-on-aDIO, this value is ignored.

    @note
        Not all aDIO devices support Wake-on-aDIO. See ReadMe.txt for more
        information on supported devices.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Interrupt_SetConfig( aDIO_Handle             Handle,
                                            BOOL                    IntEnable,
                                            aDIO_Interrupt_Int_Type Type,
                                            BOOL                    WakeEnable);

/**
    @brief
        Gets the current configuration of interrupts and Wake-on-aDIO.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        IntEnable

        A pointer to a location that will be filled with TRUE if interrupts are
        enabled or FALSE if interrupts are disabled. The actual interrupt type
        that is enabled is specified by Type.

    @param
        Type

        A pointer to a location that will be filled with the interrupt type that
        is enabled, if IntEnable was set to TRUE.

    @param
        WakeEnable

        A pointer to a location that will be filled with TRUE if Wake-on-aDIO
        is enabled or FALSE if Wake-on-aDIO is disabled or is not supported by
        the aDIO device. If Wake-on-aDIO configuration is not needed, this
        value can be NULL.

    @note
        Not all aDIO devices support Wake-on-aDIO. See ReadMe.txt for more
        information on supported devices.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Interrupt_GetConfig(
                                        aDIO_Handle                 Handle,
                                        BOOL*                       IntEnable,
                                        aDIO_Interrupt_Int_Type*    Type,
                                        BOOL*                       WakeEnable);

/**
    @brief
        Retrieves the interrupt count for a particular interrupt type.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Type

        The interrupt type to get the count of.

    @param
        Count

        A pointer to a location that will be filled with the number of times
        this interrupt type occurred since the last time this function was
        called.

    @param
        CompareValue

        A pointer to a location that will be filled with the value of the
        'Compare' register when the most recent interrupt (of any type)
        occurred. This parameter can be NULL.

    @param
        Port0Value

        A pointer to a location that will be filled with the value of Port 0
        when the most recent interrupt (of any type) occurred. This parameter
        can be NULL.

    @param
        Port1Value

        A pointer to a location that will be filled with the value of Port 1
        when the most recent interrupt (of any type) occurred. This parameter
        can be NULL.

    @warning
        This function cannot be used on interrupt types which have callbacks
        installed for them.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Interrupt_Poll(  aDIO_Handle             Handle,
                                        aDIO_Interrupt_Int_Type Type,
                                        uint32*                 Count,
                                        uint8*                  CompareValue,
                                        uint8*                  Port0Value,
                                        uint8*                  Port1Value);

/**
    @brief
        Installs a callback function for a particular interrupt type.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Type

        The interrupt type to install a callback for.

    @param
        Callback

        The callback function to be called whenever the selected interrupt
        occurs.

    @param
        UserPtr

        An arbitrary pointer that will be passed to the callback function. This
        can be used to pass user-specific context information to the callback.
        This parameter can be NULL.

    @note
        If an error occurs while the callback thread is starting, this function
        will return the appropriate error code. The callback function will NOT
        be called.

    @note
        The same callback function can be used for multiple interrupt types.

    @warning
        The given callback function will be called in its own thread context.
        The aDIOLib functions are internally synchronized, however the user is
        responsible for appropriate synchronization of any their code or data
        that is used between the main thread and any callback functions.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Interrupt_InstallCallback(
                                            aDIO_Handle             Handle,
                                            aDIO_Interrupt_Int_Type Type,
                                            aDIO_Interrupt_Callback Callback,
                                            void*                   UserPtr);

/**
    @brief
        Removes a callback function for a particular interrupt type.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Type

        The interrupt type to remove a callback for.

    @note
        If an error occurs while the callback thread is shutting down, this
        function will return the appropriate error code. The callback function
        will NOT be called.

    @note
        This function removes the callback as completely as possible, even if
        an error occurs. So, if this function does return an error, there is no
        need to call it again for the same type.

    @warning
        Do not try to remove a callback while inside the callback function.

    @warning
        Make sure the user's callback is not calling library functions when it
        is being removed or a deadlock may occur.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Interrupt_RemoveCallback(aDIO_Handle             Handle,
                                                aDIO_Interrupt_Int_Type Type);

//============================================================================
//! @} End of Interrupt Functions
//============================================================================

//============================================================================
//! @} End of Interrupt Module
//============================================================================

//============================================================================
//! @} End of API
//============================================================================


#ifdef __cplusplus
}
#endif


#endif // __ADIOLIB_INTERRUPT_H__
