/**
    @file

    @brief
        This file defines the 'Port' module of the aDIOLib API.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Port.h 55052 2011-08-16 19:33:01Z jstrouse $
*/

#ifndef __ADIOLIB_PORT_H__
#define __ADIOLIB_PORT_H__

#include "aDIOLib_General.h"

#ifdef __cplusplus
extern "C" {
#endif


//============================================================================
//! @defgroup aDIO_API aDIOLib API
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Port aDIOLib Port Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Port_Define aDIOLib Port Defines
//! @{
//============================================================================

/**
    Defines a bit mask for bit 0 of a digital IO port.
*/
#define ADIO_PORT_BIT_0     0x01

/**
    Defines a bit mask for bit 1 of a digital IO port.
*/
#define ADIO_PORT_BIT_1     0x02

/**
    Defines a bit mask for bit 2 of a digital IO port.
*/
#define ADIO_PORT_BIT_2     0x04

/**
    Defines a bit mask for bit 3 of a digital IO port.
*/
#define ADIO_PORT_BIT_3     0x08

/**
    Defines a bit mask for bit 4 of a digital IO port.
*/
#define ADIO_PORT_BIT_4     0x10

/**
    Defines a bit mask for bit 5 of a digital IO port.
*/
#define ADIO_PORT_BIT_5     0x20

/**
    Defines a bit mask for bit 6 of a digital IO port.
*/
#define ADIO_PORT_BIT_6     0x40

/**
    Defines a bit mask for bit 7 of a digital IO port.
*/
#define ADIO_PORT_BIT_7     0x80

/**
    Defines a bit mask for all 8 bits of a digital IO port.
*/
#define ADIO_PORT_BITS_ALL  0xFF

//============================================================================
//! @} End of Port Defines
//============================================================================

//============================================================================
//! @defgroup aDIO_Port_Enum aDIOLib Port Enumerations
//! @{
//============================================================================

/**
    Enumerates the two digital IO ports.
*/
typedef enum
{
    /**
        Represents digital IO port 0. (Direction is bit-programmable.)
    */
    ADIO_PORT_DIO_PORT_0 = 0,

    /**
        Represents digital IO port 1. (Direction is byte-programmable.)
    */
    ADIO_PORT_DIO_PORT_1,

} aDIO_Port_DIO_Port;

/**
    Enumerates the possible 'directions' for digital IO port 1.
*/
typedef enum
{
    /**
        Represents port 1 being an INPUT.
    */
    ADIO_PORT_DIR_P1_INPUT = 0,

    /**
        Represents port 1 being an OUTPUT.
    */
    ADIO_PORT_DIR_P1_OUTPUT,

} aDIO_Port_Dir_P1;

/**
    Enumerates the two Strobe signals.
*/
typedef enum
{
    /**
        Represents digital IO Strobe 0.
    */
    ADIO_PORT_STROBE_0 = 0,

    /**
        Represents digital IO Strobe 1.
    */
    ADIO_PORT_STROBE_1,

} aDIO_Port_Strobe;

/**
    Enumerates the digital IO Strobe levels.
*/
typedef enum
{
    /**
        Represents a 'low' strobe level.
    */
    ADIO_PORT_STROBE_LEV_LOW = 0,

    /**
        Represents a 'high' strobe level.
    */
    ADIO_PORT_STROBE_LEV_HIGH = 1,

} aDIO_Port_Strobe_Lev;

//============================================================================
//! @} End of Port Enumerations
//============================================================================

//============================================================================
//! @defgroup aDIO_Port_Func aDIOLib Port Functions
//! @{
//============================================================================

/**
    @brief
        Reads the value from a digital IO port.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Port

        The port to be read.

    @param
        Value

        A pointer to a uint8 variable which will be filled with the data from
        the digital IO port.

    @note
        Normally, this function reads the incoming value of inputs as well as
        the outgoing value of outputs. However, this function does not provide
        the value of bits on Port 0 that are set to output and are masked in the
        Mask register.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_ReadPort(   aDIO_Handle         Handle,
                                        aDIO_Port_DIO_Port  Port,
                                        uint8*              Value);

/**
    @brief
        Writes a value to a digital IO port.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Port

        The port to be written to.

    @param
        Value

        The value to write to the port.

    @note
        Only the OUTPUT bits of the port will be affected by this function.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_WritePort(  aDIO_Handle         Handle,
                                        aDIO_Port_DIO_Port  Port,
                                        uint8               Value);

/**
    @brief
        Gets the current level of a digital IO strobe.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Strobe

        The strobe to get the level/status of.

    @param
        Level

        A pointer to a aDIO_Port_Strobe_Lev variable which will be filled with
        the current level of the requested strobe.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_ReadStrobe( aDIO_Handle             Handle,
                                        aDIO_Port_Strobe        Strobe,
                                        aDIO_Port_Strobe_Lev*   Level);

/**
    @brief
        Gets the current 'direction' (input or output) of the digital IO ports.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Port0

        A pointer to a uint8 variable which will be filled with the current
        direction of port 0. If a bit's value is zero it means the corresponding
        bit of port 0 is an input. Similarly, a bit with a value of one means
        the corresponding bit of port 0 is an output.

    @param
        Port1

        A pointer to a aDIO_Port_Dir_P1 variable which will be filled with the
        current direction of port 1.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_GetDirection(   aDIO_Handle         Handle,
                                            uint8*              Port0,
                                            aDIO_Port_Dir_P1*   Port1);

/**
    @brief
        Sets the current 'direction' (input or output) of the digital IO ports.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Port0

        A uint8 value describing what direction to set each bit. If a bit's
        value is zero the corresponding bit of port 0 will be an input.
        Similarly, a bit with a value of one will make the corresponding bit of
        port 0 an output. (This value can be 'built' by ORing ADIO_PORT_BIT_*
        values together.)

    @param
        Port1

        The direction to set port 1 to.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_SetDirection(   aDIO_Handle         Handle,
                                            uint8               Port0,
                                            aDIO_Port_Dir_P1    Port1);

/**
    @brief
        Retrieves the current Mask value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Mask

        A pointer to a uint8 value which is filled with the current digital IO
        mask.

    @note
        A zero bit in the mask means the corresponding port bit is used for
        comparison. A one bit in the mask means the corresponding port bit is
        NOT used for comparison. (A mask of 0xFF will disable comparisons.)

    @note
        The mask is only used when in Event or Match IRQ modes.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_GetMask(aDIO_Handle Handle,
                                    uint8*      Mask);

/**
    @brief
        Sets the desired Mask value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Mask

        A uint8 value describing the digital IO mask to set. (This value can be
        'built' by ORing together ADIO_PORT_BIT_* values.

    @note
        A zero bit in the mask means the corresponding port bit is used for
        comparison. A one bit in the mask means the corresponding port bit is
        NOT used for comparison. (A mask of 0xFF will disable comparisons.)

    @note
        The mask is only used when in Event or Match IRQ modes.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_SetMask(aDIO_Handle Handle,
                                    uint8       Mask);

/**
    @brief
        Retrieves the current Compare value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Compare

        A pointer to a uint8 value which is filled with the current digital IO
        compare value.

    @note
        The compare value is only used, and can only be set, when in Match IRQ
        mode.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_GetCompare( aDIO_Handle Handle,
                                        uint8*      Compare);

/**
    @brief
        Sets the desired Compare value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Compare

        A uint8 value to set the digital IO compare value to.

    @note
        The compare value is only used, and can only be set, when in Match IRQ
        mode. This is the value that is checked against digital IO port 0 to
        determine when a match occurs.

*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_SetCompare( aDIO_Handle Handle,
                                        uint8       Compare);

//============================================================================
//! @} End of Port Functions
//============================================================================

//============================================================================
//! @} End of Port Module
//============================================================================

//============================================================================
//! @} End of API
//============================================================================


#ifdef __cplusplus
}
#endif

#endif // __ADIOLIB_PORT_H__
