/**
    @file

    @brief
        This file contains the IOCTL codes, structures and enumerations for
        direct driver access.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIO_Interface.h 74549 2013-11-14 19:26:44Z mmcintire $
*/


// Define the Setup Class GUID for the aDIO. This is same
// as the "RTD cpuModules" guid in the INF files.
DEFINE_GUID (GUID_DEVCLASS_ADIO,
            0xB2A369C9,0xCBD9,0x40E3,0x8A,0x22,0x66,0x26,0x6F,0xF0,0xD6,0x17);
//{B2A369C9-CBD9-40E3-8A22-66266FF0D617}



#ifndef __ADIO_INTERFACE_H__
#define __ADIO_INTERFACE_H__


#include "RTD_Types.h"
#include "aDIO_Interrupt_Types.h"

/**
    Defines the maximum number of devices (boards) that this driver will
    simultaneously support.
*/
#define ADIO_INTRFC_MAX_DEVICE_COUNT  8


/**
    Represents the availability of aDIO devices. Used in registry enumeration
    entries.
*/
typedef enum
{
    /**
        Represents an invalid device.
    */
    ADIO_INTRFC_AVAIL_INVALID = 0,

    /**
        Represents a device that is not functioning correctly.
    */
    ADIO_INTRFC_AVAIL_MALFUNC,

    /**
        Represents a device that is working but is already being used.
    */
    ADIO_INTRFC_AVAIL_IN_USE,

    /**
        Represents a device that is completely ready for use.
    */
    ADIO_INTRFC_AVAIL_READY,

} aDIO_Intrfc_Avail;




// Defines for registry enumeration key names (ANSI and Unicode)
/**
    Defines the ANSI name of the 'device number' enumeration value.
*/
#define ADIO_INTRFC_REGNAME_ANSI_DEV_NUM      "Device_Number"

/**
    Defines the UNICODE name of the 'device number' enumeration value.
*/
#define ADIO_INTRFC_REGNAME_WIDE_DEV_NUM      L"Device_Number"

/**
    Defines the ANSI name of the 'availability' enumeration value.
*/
#define ADIO_INTRFC_REGNAME_ANSI_AVAIL        "Availability"

/**
    Defines the UNICODE name of the 'availability' enumeration value.
*/
#define ADIO_INTRFC_REGNAME_WIDE_AVAIL        L"Availability"

/**
    Defines the ANSI name of the 'base address' enumeration value.
*/
#define ADIO_INTRFC_REGNAME_ANSI_BASE_ADDR    "Base_Address"

/**
    Defines the UNICODE name of the 'base address' enumeration value.
*/
#define ADIO_INTRFC_REGNAME_WIDE_BASE_ADDR    L"Base_Address"

/**
    Defines the ANSI name of the 'irq number' enumeration value.
*/
#define ADIO_INTRFC_REGNAME_ANSI_IRQ_NUM      "IRQ"

/**
    Defines the UNICODE name of the 'irq number' enumeration value.
*/
#define ADIO_INTRFC_REGNAME_WIDE_IRQ_NUM      L"IRQ"


/**
    Defines the ANSI name of the 'is wake supported' enumeration value.
*/
#define ADIO_INTRFC_REGNAME_ANSI_WAKE_SUPPORT "Is_Wake_Supported"

/**
    Defines the UNICODE name of the 'is wake supported' enumeration value.
*/
#define ADIO_INTRFC_REGNAME_WIDE_WAKE_SUPPORT L"Is_Wake_Supported"



/**
    Defines the maximum number of register blocks usable by one device.
*/
#define ADIO_INTRFC_MAX_REG_BLK_COUNT 4

/**
    A sub-structure used in the IOCTL_ADIO_QUERY_BOARD IOCTL.
*/
typedef struct
{
    /**
        This is the register block's starting (physical) address.
    */
    uint32 start_address;

    /**
        This is the register block's length in bytes.
    */
    uint32 length;

    /**
        If this value is TRUE, then this register block is memory-mapped. If
        this value is FALSE, then this register block is IO-mapped.
    */
    BOOLEAN is_memory;

} aDIO_Intrfc_Reg_Block_Info;

/**
    A structure used in the IOCTL_ADIO_QUERY_BOARD IOCTL.
*/
typedef struct
{
    /**
        This index is a value unique to this board, out of all boards
        controlled by this driver.
    */
    uint32 index;

    /**
        This value is the total number of boards being controlled by this
        driver.
    */
    uint32 board_total;

    /**
        This is an array describing the register blocks being used by this
        board.
    */
    aDIO_Intrfc_Reg_Block_Info reg_blocks[ADIO_INTRFC_MAX_REG_BLK_COUNT];

    /**
        This value is the number of the interrupt that the board is using.
    */
    uint32 irq_number;


    /**
        This value marks whether or not the aDIO device supports Wake-on-aDIO.
    */
    BOOLEAN is_wake_supported;

} aDIO_Intrfc_Board_Info;

/**
    Describes the possible register operation types.
*/
typedef enum
{
    /**
        Reads the register at 'offset' and returns it in 'data'.
    */
    ADIO_INTRFC_REG_OP_READ = 0,

    /**
        Writes the value in 'data' to the register at 'offset'.
    */
    ADIO_INTRFC_REG_OP_WRITE,

    /**
        Writes the value in 'data' to the register at 'offset', only changing
        the bits denoted by 'mask'.

        @note
            newValue = (oldValue & ~mask) | (data & mask)
    */
    ADIO_INTRFC_REG_OP_SAFEWRITE,

} aDIO_Intrfc_Reg_Op_Type;

/**
    Describes the possible register operation sizes.
*/
typedef enum
{
    /**
        Represents an 8-bit operation.
    */
    ADIO_INTRFC_REG_SIZE_8 = 1,



} aDIO_Intrfc_Reg_Op_Size;

/**
    A structure used in the IOCTL_ADIO_REGISTER_OP IOCTL.
*/
typedef struct
{
    /**
        This is the particular operation to perform.
    */
    aDIO_Intrfc_Reg_Op_Type op;

    /**
        This is the size of the register operation to perform.
    */
    aDIO_Intrfc_Reg_Op_Size size;

    /**
        This value indicates which register block that this operation is to
        occur in. The value is zero-based.
    */
    uint8 block;

    /**
        This is an offset into the register block, indicating which register to
        perform the operation on.
    */
    uint32 offset;

    /**
        This holds an incoming or outgoing register value. (On an 8-bit
        operation only the first byte is used. On a 16-bit operation only the
        first two bytes are used.)
    */
    uint32 data;

    /**
        This is the mask value to use in a ADIO_INTRFC_REG_OP_SAFEWRITE
        operation. (On an 8-bit operation only the first byte is used. On a
        16-bit operation only the first two bytes are used.)

        A one in this value means that the corresponding bit in the register CAN
        be altered. (mask = 0xFF -> same as a WRITE; mask = 0x00 -> no change to
        the register)
    */
    uint32 mask;

} aDIO_Intrfc_Reg_Op;

// Guarded bits
/**
    Enumerates 'guarded' bitfields.

    Guarded bitfields are guaranteed to be saved and restored when used inside
    the driver. They can be locked to a particular value (and later unlocked)
    via the IOCTL_ADIO_GUARD_BIT_CONTROL IOCTL.

    @note This enumeration must match ADIO_INTRFC_GUARDED_TABLE.
*/
typedef enum
{
    /**
        This value represents the 'Multi-Function Register Select' bitfield of
        the 'DIO Control' register.
    */
    ADIO_INTRFC_GUARDED_TYPE_MF_SELECT,

    /**
        Equals the number of guarded bitfields.
    */
    ADIO_INTRFC_GUARDED_TYPE_COUNT

} aDIO_Intrfc_Guarded_Type;

/**
    Describes the 'guarded' bitfields enumerated by aDIO_Intrfc_Guarded_Type.

    Each entry in the table is an array of ULONGS. Each array has the following
    data: the register operation size (in bytes) needed to access the bitfield,
    the bitfield's register block, its offset, and a bitmask giving the
    particular bits that make up the bitfield.

    @note This table must match aDIO_Intrfc_Guarded_Type.
*/
#define ADIO_INTRFC_GUARDED_TABLE {{1, 0, 3, 0x3}}

/**
    A structure used in the IOCTL_ADIO_GUARD_BIT_CONTROL IOCTL.
*/
typedef struct
{
    /**
        This is the 'guarded' bitfield to lock or unlock.
    */
    aDIO_Intrfc_Guarded_Type guard_bits;

    /**
        This value determines whether the bitfield is to be locked or unlocked.

        If this value is TRUE, an attempt will be made to lock the bitfield. If
        the bitfield is already locked, the status STATUS_INVALID_DEVICE_REQUEST
        will be returned from the IOCTL.

        If this value is FALSE, the bitfield will be unlocked. Attempting to
        unlock a bitfield that is not locked will do nothing and simply return
        STATUS_SUCCESS.
    */
    BOOLEAN do_lock;

    /**
        If do_lock is TRUE, this is the value to lock the bitfield into
        otherwise this value is ignored.

        If locking, the register containing the bitfield will be changed as
        follows (like a safe-write register op):
        field_register = (field_register & ~field_mask) | (data & field_mask)
    */
    uint32 data;

} aDIO_Intrfc_Guard_Bit_Control;


/**
    A structure used in the IOCTL_ADIO_INT_CONTROL IOCTL.
*/
typedef struct
{
    /**
        This is the type of interrupt to control.
    */
    aDIO_Interrupt_Type type;

    /**
        Pass in TRUE to set the interrupt to the state given by 'enable'. Pass
        in FALSE to simply fill 'enable' with the interrupt's current
        enable/disable state.
    */
    BOOLEAN change_state;

    /**
        When this value is TRUE it represents an enabled interrupt. When this
        value is FALSE it represents a disabled interrupt. See the description
        of 'change_state' for details.

        @note
            Enabling an interrupt will clear it's 'count' and 'overrun count'.

        @note
            Since all interrupt types are mutually exclusive, enabling any
            interrupt will automatically disable all other interrupts.
    */
    BOOLEAN enable;

} aDIO_Intrfc_Int_Control;

/**
    Describes the possible interrupt status operation types.
*/
typedef enum
{
    /**
        Represents a simple polling of interrupt status.

        @note
            When using this operation, DeviceIoControl completes immediately and
            the structure given as output is valid at that time.
    */
    ADIO_INTRFC_INT_STATUS_OP_POLL,

    /**
        Represents a request for a notification the next time a certain
        interrupt occurs.

        @note
            To use this operation, the file used to access the device must have
            been opened with 'overlapped' capabilities. Also the DeviceIoControl
            call must have been given an OVERLAPPED structure, containing a
            valid, unique event handle.

        @note
            When using this operation, DeviceIoControl completes immediately but
            the structure given as output is not yet valid. The event in the
            OVERLAPPED structure will be signaled (to notify the user) the next
            time the relevant event occurs (an interrupt of a specific type).
            The structure given as output will be valid at this time.
    */
    ADIO_INTRFC_INT_STATUS_OP_NOTIFY,

    /**
        Represents clearing the notification request queue for a certain
        interrupt type by canceling all of the interrupt type's notification
        requests.

        The notification requests cleared from the queue will NOT have a valid
        aDIO_Intrfc_Int_Status structure upon cancellation. Also, if
        GetOverlappedResult is called for a canceled request it will return an
        error and GetLastError will equal ERROR_OPERATION_ABORTED.

        @note
            When using this operation, DeviceIoControl completes immediately and
            the structure given as output is valid at that time.
    */
    ADIO_INTRFC_INT_STATUS_OP_CLEAR_QUEUE

} aDIO_Intrfc_Int_Status_Op;

/**
    A structure used in the IOCTL_ADIO_INT_STATUS IOCTL.
*/
typedef struct
{
    /**
        This is the type of interrupt to get the status of.
    */
    aDIO_Interrupt_Type type;

    /**
        This is the status operation to be performed.
    */
    aDIO_Intrfc_Int_Status_Op op;

    /**
        When the output structure is valid (see aDIO_Intrfc_Int_Status_Op)
        this value contains the interrupt count for the specified interrupt.

        @note
            An interrupt's count is reset every time you: check its status using
            ADIO_INTRFC_INT_STATUS_OP_POLL, get notification of it using
            ADIO_INTRFC_INT_STATUS_OP_NOTIFY, or enable it using
            IOCTL_ADIO_INT_CONTROL.

        @note
            If 'op' is set to ADIO_INTRFC_INT_STATUS_OP_CLEAR_QUEUE, this
            value is not used.
    */
    uint32 count;

    /**
        Holds the value of the 'Compare' register at the time of the last
        interrupt.

        @note
            When a Strobe or Event interrupt occurs, Port 0 is latched into the
            'Compare' register.
    */
    uint8 last_compare_val;

    /**
        Holds the value of Port 0 at the time of the last interrupt.

        @note
            This value is not latched, it is simply read during interrupt
            processing. This means there is a short interval in which the value
            of Port 0 could change from its value at the time of the interrupt.
    */
    uint8 last_port0_val;

    /**
        Holds the value of Port 1 at the time of the last interrupt.

        @note
            This value is not latched, it is simply read during interrupt
            processing. This means there is a short interval in which the value
            of Port 1 could change from its value at the time of the interrupt.
    */
    uint8 last_port1_val;

} aDIO_Intrfc_Int_Status;


/**
    A structure used in the IOCTL_ADIO_WAKE_CONTROL IOCTL.
*/
typedef struct
{
    /**
        Marks if the enable/disable state of Wake-on-aDIO should be changed or
        simply retrieved. Pass in TRUE to set Wake-on-aDIO to the state given by
        'enable'. Pass in FALSE to simply fill 'enable' with Wake-on-aDIO's
        current state.
    */
    BOOLEAN change_enable;

    /**
        Marks either the enable/disable state to put Wake-on-aDIO into or the
        current enable/disable state of Wake-on-aDIO. See 'change_enable' for
        details.
    */
    BOOLEAN enable;

} aDIO_Intrfc_Wake_Control;


/**
    Defines a macro for creating new IOCTLs.
*/
#define ADIO_NEW_IOCTL(v) CTL_CODE(   0x8000,\
                                        v,\
                                        METHOD_BUFFERED,\
                                        FILE_READ_ACCESS | FILE_WRITE_ACCESS)

/**
    @brief
        Defines a IOCTL to query the board about its configuration.

    The output buffer must be a aDIO_Intrfc_Board_Info structure. The input
    buffer is not used.

    @note
        If this device's file was opened with the FILE_FLAG_OVERLAPPED flag,
        then any calls to DeviceIoControl must have a valid OVERLAPPED structure
        (with valid hEvent parameter) passed in via the lpOverlapped parameter.

*/
#define IOCTL_ADIO_QUERY_BOARD        ADIO_NEW_IOCTL(0x800)

/**
    @brief
        Defines a IOCTL to 'reset' the board and any associated state inside
        the driver.

    The input and output buffers are not used.

    @note
        When this IOCTL is used, all guarded bitfields will be forcibly
        unlocked.

    @note
        If this device's file was opened with the FILE_FLAG_OVERLAPPED flag,
        then any calls to DeviceIoControl must have a valid OVERLAPPED structure
        (with valid hEvent parameter) passed in via the lpOverlapped parameter.

*/
#define IOCTL_ADIO_BOARD_RESET        ADIO_NEW_IOCTL(0x801)

/**
    @brief
        Defines a IOCTL to read, write or safe-write an aDIO register.

    The input and output buffers must both be a aDIO_Intrfc_Reg_Op structure.

    @note
        If this device's file was opened with the FILE_FLAG_OVERLAPPED flag,
        then any calls to DeviceIoControl must have a valid OVERLAPPED structure
        (with valid hEvent parameter) passed in via the lpOverlapped parameter.

*/
#define IOCTL_ADIO_REGISTER_OP        ADIO_NEW_IOCTL(0x810)

/**
    @brief
        Defines a IOCTL to lock or unlock 'guarded' bitfields.

    The input and output buffers must both be a aDIO_Intrfc_Guard_Lock
    structure.

    @note
        If this device's file was opened with the FILE_FLAG_OVERLAPPED flag,
        then any calls to DeviceIoControl must have a valid OVERLAPPED structure
        (with valid hEvent parameter) passed in via the lpOverlapped parameter.

*/
#define IOCTL_ADIO_GUARD_BIT_CONTROL  ADIO_NEW_IOCTL(0x811)

/**
    @brief
        Defines a IOCTL to disable/enable interrupts and retrieve their
        disable/enable state.

    The input and output buffers must both be a aDIO_Intrfc_Int_Control
    structure.

    @note
        If this device's file was opened with the FILE_FLAG_OVERLAPPED flag,
        then any calls to DeviceIoControl must have a valid OVERLAPPED structure
        (with valid hEvent parameter) passed in via the lpOverlapped parameter.

*/
#define IOCTL_ADIO_INT_CONTROL        ADIO_NEW_IOCTL(0x820)

/**
    @brief
        Defines a IOCTL which can retrieve the status of an interrupt or can
        ask the driver to signal an event the next time the interrupt happens.

    The input and output buffers must both be a aDIO_Intrfc_Int_Status
    structure.

    @note
        This IOCTL can be used on a single interrupt type more than once
        provided that the user's notification events (the events passed in to
        DeviceIoControl via the OVERLAPPED structure) are unique. If this is
        done, the events will be queued in the order of their arrival.

    @note
        If this device's file was opened with the FILE_FLAG_OVERLAPPED flag,
        then any calls to DeviceIoControl must have a valid OVERLAPPED structure
        (with valid hEvent parameter) passed in via the lpOverlapped parameter.

*/
#define IOCTL_ADIO_INT_STATUS         ADIO_NEW_IOCTL(0x821)


/**
    @brief
        Defines a IOCTL to disable/enable Wake-on-aDIO and retrieve its
        disable/enable state.

    The input and output buffers must both be a aDIO_Intrfc_Wake_Control
    structure.

    @note
        If this device's file was opened with the FILE_FLAG_OVERLAPPED flag,
        then any calls to DeviceIoControl must have a valid OVERLAPPED structure
        (with valid hEvent parameter) passed in via the lpOverlapped parameter.

*/
#define IOCTL_ADIO_WAKE_CONTROL       ADIO_NEW_IOCTL(0x840)

#endif //__ADIO_INTERFACE_H__
