/**
    @file

    @brief
        This file implements the 'General' module of the aDIOLib API.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_General.c 55027 2011-08-15 20:24:09Z jstrouse $
*/

#include "aDIOLib_General.h"
#include "aDIOLib_General_Internal.h"
#include "aDIOLib_Interrupt_Internal.h"
#include "aDIOLib_Log_Internal.h"


//============================================================================
// aDIOLib API
//============================================================================

//============================================================================
// aDIOLib General Module
//============================================================================

//============================================================================
// aDIOLib General Functions
//============================================================================

/*
    Enumerates the aDIO devices currently installed in the system.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_General_Enum(aDIO_Enum_Info* DeviceList,
                                    uint8           ListSize)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    LONG err_val = ERROR_SUCCESS;
    HKEY enum_key;
    DWORD i;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_PTR(NULL,
                                    DeviceList,
                                    ADIO_ERROR_CATEGORY_GENERAL);
    if(ListSize <= 0)
    {
        return ADIO_LOG_INTERNAL_LOG_BAD_PARAM_INT( NULL,
                                                    ADIO_ERROR_CATEGORY_GENERAL,
                                                    ListSize);
    }

    // Mark all enum structures as invalid, set 0 values
    for(i = 0; i < ListSize; ++i)
    {
        DeviceList[i].availability = ADIO_INTRFC_AVAIL_INVALID;
        DeviceList[i].dev_num = 0;
        DeviceList[i].base_address = 0;
        DeviceList[i].irq = 0;
        DeviceList[i].has_wake = FALSE;
    }

    // Open the overall enum key, which has the device keys under it
    err_val = RegOpenKeyEx( HKEY_LOCAL_MACHINE,
                            "SYSTEM\\CurrentControlSet\\Services\\aDIO\\Enum",
                            0,
                            KEY_READ,
                            &enum_key);
    if(err_val != ERROR_SUCCESS)
    {
        return ADIO_LOG_INTERNAL_LOG_WIN_API(   NULL,
                                                ADIO_ERROR_CATEGORY_GENERAL,
                                                "RegOpenKeyEx");
    }

    // Enumerate the device sub-keys
    for(i = 0; i < ListSize; ++i)
    {
        result = aDIO_General_Internal_EnumReadKey( enum_key,
                                                    i,
                                                    &(DeviceList[i]));
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;
    }

    // Close the registry key
    err_val = RegCloseKey(enum_key);
    if(err_val != ERROR_SUCCESS)
    {
        SetLastError(err_val);
        ADIO_LOG_INTERNAL_LOG_WIN_API(  NULL,
                                        ADIO_ERROR_CATEGORY_GENERAL,
                                        "RegCloseKey");
        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = ADIO_ERROR_CODE_WIN_API;
    }

    return result;
}

/*
    Opens an aDIO device and readies it for use.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_General_Open(aDIO_Enum_Info  DeviceInfo,
                                    aDIO_Handle*    Handle)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Handle new_handle = NULL;
    char dev_name[20];

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_PTR(NULL, Handle, ADIO_ERROR_CATEGORY_GENERAL);
    if(DeviceInfo.availability != ADIO_INTRFC_AVAIL_READY)
    {
        return ADIO_LOG_INTERNAL_LOG_IN_USE(NULL,
                                            ADIO_ERROR_CATEGORY_GENERAL,
                                            "The aDIO being opened");
    }

    // Allocate memory for the aDIO descriptor
    new_handle = (aDIO_Handle)malloc(sizeof(aDIO_Descriptor));
    if(!new_handle)
    {
        return ADIO_LOG_INTERNAL_LOG_NO_MEMORY( NULL,
                                                ADIO_ERROR_CATEGORY_GENERAL,
                                                sizeof(aDIO_Descriptor));
    }

    // Set entire descriptor to NULL (zero)
    memset(new_handle, '\0', sizeof(aDIO_Descriptor));

    // Initialize and enter the global critical section
    InitializeCriticalSection(&new_handle->global_lock);
    EnterCriticalSection(&new_handle->global_lock);

    // Initialize the logging function's critical section now, in case an error
    //  occurs during opening
    InitializeCriticalSection(&new_handle->log_section);

    do
    {
        // Create the name for the device file to open
        if(sprintf_s(   dev_name,
                        20,
                        "\\\\.\\RTD_aDIO_%d",
                        DeviceInfo.dev_num) <= 0)
        {
            result = ADIO_LOG_INTERNAL_LOG_UNEXPECTED(
                                                NULL,
                                                ADIO_ERROR_CATEGORY_GENERAL,
                                                "Call to sprintf_s failed.");
            break;
        }

        // Open the device file
        new_handle->dev_file = CreateFile(  dev_name,
                                            GENERIC_READ | GENERIC_WRITE,
                                            0,
                                            NULL,
                                            OPEN_EXISTING,
                                            FILE_FLAG_OVERLAPPED,
                                            0);
        if(new_handle->dev_file == INVALID_HANDLE_VALUE)
        {
            result = ADIO_LOG_INTERNAL_LOG_WIN_API( NULL,
                                                    ADIO_ERROR_CATEGORY_GENERAL,
                                                    "CreateFile");
            break;
        }

        do
        {
            // Query the device's configuration
            result = aDIO_General_Internal_CallIoctl(
                                                new_handle,
                                                IOCTL_ADIO_QUERY_BOARD,
                                                &(new_handle->dev_info),
                                                sizeof(aDIO_Intrfc_Board_Info),
                                                NULL);
            if(result != ADIO_ERROR_CODE_SUCCESS)
                break;

            // Prepare the Interrupt module
            result = aDIO_Interrupt_Internal_PrepareModule(new_handle);
            if(result != ADIO_ERROR_CODE_SUCCESS)
                break;

            // Opening is complete. Fill in the user's handle and leave the
            //  global critical section before returning
            *Handle = new_handle;

            LeaveCriticalSection(&new_handle->global_lock);

        } while(0);

        // If there was a failure, cleanup the modules and the device file
        if(result != ADIO_ERROR_CODE_SUCCESS)
        {
            aDIO_Interrupt_Internal_CleanModule(new_handle);

            if(!CloseHandle(new_handle->dev_file))
            {
                ADIO_LOG_INTERNAL_LOG_WIN_API(  NULL,
                                                ADIO_ERROR_CATEGORY_GENERAL,
                                                "CloseHandle");
            }
        }

    } while(0);

    // There was a failure: leave the global critical section, delete both
    //  critical sections and free the descriptor memory
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        DeleteCriticalSection(&new_handle->log_section);
        LeaveCriticalSection(&new_handle->global_lock);
        DeleteCriticalSection(&new_handle->global_lock);
        free(new_handle);
    }

    return result;
}

/*
    Closes an aDIO device and performs needed cleanup.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_General_Close(aDIO_Handle Handle)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result = ADIO_ERROR_CODE_SUCCESS;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_GENERAL);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    // Clean up the Interrupt module
    temp_result = aDIO_Interrupt_Internal_CleanModule(Handle);
    if(result == ADIO_ERROR_CODE_SUCCESS)
        result = temp_result;

    // Close the aDIO device file
    if(!CloseHandle(Handle->dev_file))
    {
        ADIO_LOG_INTERNAL_LOG_WIN_API(  NULL,
                                        ADIO_ERROR_CATEGORY_GENERAL,
                                        "CloseHandle");
        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = ADIO_ERROR_CODE_WIN_API;
    }

    // Delete the logging function's critical section
    DeleteCriticalSection(&Handle->log_section);

    // Release the global lock and delete it
    LeaveCriticalSection(&Handle->global_lock);
    DeleteCriticalSection(&Handle->global_lock);

    // Free the memory allocated for the descriptor
    free(Handle);

    return result;
}

/*
    Resets the aDIO circuitry to a 'default' state. (All registers are made
    zero.)
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_General_Reset(aDIO_Handle Handle)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_GENERAL);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    do
    {
        // Reset the Interrupt module
        result = aDIO_Interrupt_Internal_ResetModule(Handle);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Perform the reset
        result = aDIO_General_Internal_CallIoctl(   Handle,
                                                    IOCTL_ADIO_BOARD_RESET,
                                                    NULL,
                                                    0,
                                                    NULL);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

    } while(0);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Retrieves the error message recorded for an error returned from the aDIOLib
    API.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_General_GetLastErrorMsg( aDIO_Handle Handle,
                                                char*       MessageBuffer,
                                                uint32      BufferSize)
{
    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_GENERAL);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(NULL,
                                    MessageBuffer,
                                    ADIO_ERROR_CATEGORY_GENERAL);
    if(BufferSize < ADIO_ERROR_MAX_MSG_LENGTH)
    {
        return ADIO_LOG_INTERNAL_LOG_BAD_PARAM_INT( Handle,
                                                    ADIO_ERROR_CATEGORY_GENERAL,
                                                    BufferSize);
    }

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    EnterCriticalSection(&Handle->log_section);

    // If there is an error message, copy it out and 'clear' it
    if(Handle->last_error_msg[0] != '\0')
    {
        strcpy_s(MessageBuffer, BufferSize, Handle->last_error_msg);

        Handle->last_error_msg[0] = '\0';
    }

    LeaveCriticalSection(&Handle->log_section);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return ADIO_ERROR_CODE_SUCCESS;
}

//============================================================================
// End of General Functions
//============================================================================

//============================================================================
// End of General Module
//============================================================================

//============================================================================
// End of API
//============================================================================

