/**
    @file

    @brief
        This file implements internal functions for the 'General' module of the
        aDIOLib.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_General_Internal.c 55027 2011-08-15 20:24:09Z jstrouse $
*/

#include "aDIOLib_General_Internal.h"
#include "aDIOLib_Log_Internal.h"
#include "aDIOLib_RegMap_Internal.h"


//============================================================================
// aDIOLib Internal Module
//============================================================================

//============================================================================
// aDIOLib General Internal Module
//============================================================================

//============================================================================
// aDIOLib General Internal Functions
//============================================================================

/*
    Reads enumeration information out of a device's registry key.
*/
aDIO_Error aDIO_General_Internal_EnumReadKey(   HKEY            EnumKey,
                                                DWORD           Index,
                                                aDIO_Enum_Info* DevInfo)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    LONG err_val = ERROR_SUCCESS;
    TCHAR sub_key_name[255];
    DWORD max_key_len = 255;
    DWORD value_type, value_data;
    DWORD value_size = sizeof(DWORD);
    FILETIME last_write_time;
    HKEY sub_key;

    err_val = RegEnumKeyEx( EnumKey,
                            Index,
                            sub_key_name,
                            &max_key_len,
                            NULL,
                            NULL,
                            NULL,
                            &last_write_time);
    if(err_val == ERROR_NO_MORE_ITEMS)
    {
        return ADIO_ERROR_CODE_SUCCESS;
    }
    else if(err_val != ERROR_SUCCESS)
    {
        return ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        NULL,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "RegEnumKeyEx");
    }

    // There is another sub-key, so open it
    err_val = RegOpenKeyEx( EnumKey,
                            sub_key_name,
                            0,
                            KEY_READ,
                            &sub_key);
    if(err_val != ERROR_SUCCESS)
    {
        return ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        NULL,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "RegOpenKeyEx");
    }

    // Read the key's values
    do
    {
        // Get the device number
        err_val = RegQueryValueEx(  sub_key,
                                    ADIO_INTRFC_REGNAME_ANSI_DEV_NUM,
                                    NULL,
                                    &value_type,
                                    (LPBYTE)&value_data,
                                    &value_size);
        if( (err_val != ERROR_SUCCESS)  ||
            (value_type != REG_DWORD)   ||
            (value_size != sizeof(DWORD)))
        {
            result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        NULL,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "RegQueryValueEx");
            break;
        }
        else
        {
            DevInfo->dev_num = (uint32)value_data;
        }

        // Get the availability
        err_val = RegQueryValueEx(  sub_key,
                                    ADIO_INTRFC_REGNAME_ANSI_AVAIL,
                                    NULL,
                                    &value_type,
                                    (LPBYTE)&value_data,
                                    &value_size);
        if( (err_val != ERROR_SUCCESS)  ||
            (value_type != REG_DWORD)   ||
            (value_size != sizeof(DWORD)))
        {
            result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        NULL,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "RegQueryValueEx");
            break;
        }
        else
        {
            DevInfo->availability = (aDIO_Intrfc_Avail)value_data;
        }

        // Get the base address
        err_val = RegQueryValueEx(  sub_key,
                                    ADIO_INTRFC_REGNAME_ANSI_BASE_ADDR,
                                    NULL,
                                    &value_type,
                                    (LPBYTE)&value_data,
                                    &value_size);
        if( (err_val != ERROR_SUCCESS)  ||
            (value_type != REG_DWORD)   ||
            (value_size != sizeof(DWORD)))
        {
            result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        NULL,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "RegQueryValueEx");
            break;
        }
        else
        {
            DevInfo->base_address = (uint32)value_data;
        }

        // Get the IRQ
        err_val = RegQueryValueEx(  sub_key,
                                    ADIO_INTRFC_REGNAME_ANSI_IRQ_NUM,
                                    NULL,
                                    &value_type,
                                    (LPBYTE)&value_data,
                                    &value_size);
        if( (err_val != ERROR_SUCCESS)  ||
            (value_type != REG_DWORD)   ||
            (value_size != sizeof(DWORD)))
        {
            result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        NULL,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "RegQueryValueEx");
            break;
        }
        else
        {
            DevInfo->irq = (uint32)value_data;
        }

        // Get whether or not the aDIO supports Wake-on-aDIO
        err_val = RegQueryValueEx(  sub_key,
                                    ADIO_INTRFC_REGNAME_ANSI_WAKE_SUPPORT,
                                    NULL,
                                    &value_type,
                                    (LPBYTE)&value_data,
                                    &value_size);
        if( (err_val != ERROR_SUCCESS)  ||
            (value_type != REG_DWORD)   ||
            (value_size != sizeof(DWORD)))
        {
            result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        NULL,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "RegQueryValueEx");
            break;
        }
        else
        {
            if(value_data)
                DevInfo->has_wake = TRUE;
            else
                DevInfo->has_wake = FALSE;
        }

    } while(0);

    // Close the key
    err_val = RegCloseKey(sub_key);
    if(err_val != ERROR_SUCCESS)
    {
        ADIO_LOG_INTERNAL_LOG_WIN_API(  NULL,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "RegCloseKey");
        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = ADIO_ERROR_CODE_WIN_API;
    }

    return result;
}

/*
    Calls a device IOCTL on a device's driver.
*/
aDIO_Error aDIO_General_Internal_CallIoctl( aDIO_Handle     Handle,
                                            DWORD           IoctlCode,
                                            LPVOID          Buffer,
                                            DWORD           BufferSize,
                                            LPOVERLAPPED    Overlapped)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    char ioctl_name[ADIO_LOG_INTERNAL_MAX_IOCTL_NAME_LENGTH];
    OVERLAPPED overlapped;
    HANDLE temp_handle;
    DWORD num, res;

    // If an overlapped structure was not provided, create an event and fill
    //  the overlapped structure. If one was provided, 'reset' it.
    if(!Overlapped)
    {
        overlapped.hEvent = CreateEvent(NULL, FALSE, FALSE, "");
        if(!overlapped.hEvent)
        {
            return ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "CreateEvent");
        }

        Overlapped = &overlapped;
    }
    else
    {
        temp_handle = Overlapped->hEvent;
        memset(Overlapped, 0, sizeof(OVERLAPPED));
        Overlapped->hEvent = temp_handle;
    }

    do
    {
        // Perform the IOCTL call
        if(!DeviceIoControl(Handle->dev_file,
                            IoctlCode,
                            Buffer,
                            BufferSize,
                            Buffer,
                            BufferSize,
                            &num,
                            Overlapped))
        {
            res = GetLastError();
            if(res != ERROR_IO_PENDING)
            {
                SetLastError(res);
                result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "DeviceIoControl");
                break;
            }
        }

        // If a local event was created, wait for it to be signaled
        if(Overlapped == &overlapped)
        {
            res = WaitForSingleObject(  overlapped.hEvent,
                                        ADIO_GENERAL_INTERNAL_IOCTL_WAIT_TIME);
            if(res == WAIT_TIMEOUT)
            {
                aDIO_Log_Internal_GetIoctlName(
                                    IoctlCode,
                                    ioctl_name,
                                    ADIO_LOG_INTERNAL_MAX_IOCTL_NAME_LENGTH);
                result = ADIO_LOG_INTERNAL_LOG_ANY(
                                    Handle,
                                    ADIO_ERROR_CODE_UNEXPECTED,
                                    ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                    ADIO_ERROR_SEVERITY_ERROR,
                                    "Timed out while waiting for %s to finish.",
                                    ioctl_name);
                break;
            }
            else if(res != WAIT_OBJECT_0)
            {
                result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "WaitForSingleObject");
                break;
            }

            if(!GetOverlappedResult(Handle->dev_file,
                                    &overlapped,
                                    &num,
                                    FALSE))
            {
                aDIO_Log_Internal_GetIoctlName(
                                    IoctlCode,
                                    ioctl_name,
                                    ADIO_LOG_INTERNAL_MAX_IOCTL_NAME_LENGTH);
                result = ADIO_LOG_INTERNAL_LOG_IOCTL_FAIL(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        ioctl_name);
                break;
            }
        }

    } while(0);

    // If a local event was created earlier, release it
    if(Overlapped == &overlapped)
    {
        if(!CloseHandle(overlapped.hEvent))
        {
            ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_GENERAL_INTERNAL,
                                        "CloseHandle");

            if(result == ADIO_ERROR_CODE_SUCCESS)
                result = ADIO_ERROR_CODE_WIN_API;
        }
    }

    return result;
}

/*
    Read, write or modify a register.
*/
aDIO_Error aDIO_General_Internal_RegOp( aDIO_Handle             Handle,
                                        aDIO_Intrfc_Reg_Op_Type Type,
                                        aDIO_Intrfc_Reg_Op_Size Size,
                                        uint8                   Block,
                                        uint32                  Offset,
                                        uint32*                 Data,
                                        uint32                  Mask)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Intrfc_Reg_Op reg_op;

    // Ready the IOCTL parameters
    reg_op.op = Type;
    reg_op.size = Size;
    reg_op.block = Block;
    reg_op.offset = Offset;
    reg_op.data = *Data;
    reg_op.mask = Mask;

    // Call the IOCTL
    result = aDIO_General_Internal_CallIoctl(   Handle,
                                                IOCTL_ADIO_REGISTER_OP,
                                                &reg_op,
                                                sizeof(aDIO_Intrfc_Reg_Op),
                                                NULL);

    // If successful, copy out any data that may have been read
    if(result == ADIO_ERROR_CODE_SUCCESS)
        *Data = reg_op.data;

    return result;
}

/*
    Controls locking of the 'Multi-Function' register to a particular mode.
*/
aDIO_Error aDIO_General_Internal_LockMultiFunction(
                                aDIO_Handle                             Handle,
                                BOOL                                    Lock,
                                aDIO_General_Internal_MultiFunc_Mode    Mode)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Intrfc_Guard_Bit_Control bit_control;

    // Ready the IOCTL parameters
    bit_control.guard_bits = ADIO_INTRFC_GUARDED_TYPE_MF_SELECT;
    bit_control.do_lock = Lock;
    bit_control.data = 0;

    if(Lock)
    {
        switch(Mode)
        {
        case ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_P0DIR:
            bit_control.data = ADIO_REGMAP_INTERNAL_DIOCONTROL_MFSELECT_P0DIR;
            break;
        case ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_MASK:
            bit_control.data = ADIO_REGMAP_INTERNAL_DIOCONTROL_MFSELECT_MASKREG;
            break;
        case ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_COMPARE:
            bit_control.data = ADIO_REGMAP_INTERNAL_DIOCONTROL_MFSELECT_COMPARE;
            break;
        default:
            bit_control.data = 0;
        }
    }

    // Call the IOCTL
    result = aDIO_General_Internal_CallIoctl(
                                        Handle,
                                        IOCTL_ADIO_GUARD_BIT_CONTROL,
                                        &bit_control,
                                        sizeof(aDIO_Intrfc_Guard_Bit_Control),
                                        NULL);

    return result;
}

/*
    If the DLL is loading, register it as a source of Event log messages.
*/
BOOL APIENTRY DllMain(  HANDLE hModule,
                        DWORD  ul_reason_for_call,
                        LPVOID lpReserved)
{
    if(ul_reason_for_call != DLL_PROCESS_ATTACH)
        return TRUE;

    return aDIO_Log_Internal_RegisterEventSource(hModule);
}

//============================================================================
// End of General Internal Functions
//============================================================================

//============================================================================
// End of General Internal Module
//============================================================================

//============================================================================
// End of Internal Module
//============================================================================

