/**
    @file

    @brief
        This file defines internal functions for the 'General' module of the
        aDIOLib.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_General_Internal.h 55027 2011-08-15 20:24:09Z jstrouse $
*/

#ifndef __ADIOLIB_GENERAL_INTERNAL_H__
#define __ADIOLIB_GENERAL_INTERNAL_H__

#include "aDIOLib_General.h"

#ifdef __cplusplus
extern "C" {
#endif

//============================================================================
//! @defgroup aDIO_Internal aDIOLib Internal Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_General_Internal aDIOLib General Internal Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_General_Internal_Defines aDIOLib General Internal Defines
//! @{
//============================================================================

/**
    Defines a timeout (in milliseconds) to wait for an IOCTL to complete.
*/
#define ADIO_GENERAL_INTERNAL_IOCTL_WAIT_TIME 500

//============================================================================
//! @} End of General Internal Defines
//============================================================================

//============================================================================
//! @defgroup aDIO_General_Internal_Enum aDIOLib General Internal Enumerations
//! @{
//============================================================================

/**
    Describes the possible modes for the 'Multi-Function' register.
*/
typedef enum
{
    /**
        Represents the 'Port 0 Direction' mode of the 'Mult-Function' register.
    */
    ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_P0DIR,

    /**
        Represents the 'Mask Register' mode of the 'Mult-Function' register.
    */
    ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_MASK,

    /**
        Represents the 'Compare register' mode of the 'Mult-Function' register.
    */
    ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_COMPARE,

} aDIO_General_Internal_MultiFunc_Mode;

//============================================================================
//! @} End of General Internal Enumerations
//============================================================================

//============================================================================
//! @defgroup aDIO_General_Internal_Func aDIOLib General Internal Functions
//! @{
//============================================================================

/**
    @internal

    @brief
        Reads enumeration information out of a device's registry key.

    @param
        EnumKey

        A handle to the 'Enum' key (under which each device places their
        enumeration key).

    @param
        Index

        A zero-based offset used to iterate through the sub-keys of 'EnumKey'.

    @param
        DevInfo

        A pointer to an enumeration structure. This structure is filled with the
        information retrieved from the current sub-key.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_General_Internal_EnumReadKey(   HKEY            EnumKey,
                                                DWORD           Index,
                                                aDIO_Enum_Info* DevInfo);

/**
    @internal

    @brief
        Calls a device IOCTL on a device's driver.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        IoctlCode

        The code representing the IOCTL to be called.

    @param
        Buffer

        A pointer to a buffer to be passed to the IOCTL. This is usually a
        pointer to an aDIO_Intrfc_* structure containing the parameters for
        the IOCTL.

    @param
        BufferSize

        The size, in bytes, of the buffer pointed to by 'Buffer'.

    @param
        Overlapped

        A pointer to an overlapped structure to use while calling the IOCTL.
        This overlapped structure must contain a valid event. If it does, the
        function will return immediately and the event will be signaled when the
        IOCTL completes. Alternatively, this value can be NULL. In this case an
        overlapped structure will be used internally and this function will not
        return until the IOCTL completes.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_General_Internal_CallIoctl( aDIO_Handle     Handle,
                                            DWORD           IoctlCode,
                                            LPVOID          Buffer,
                                            DWORD           BufferSize,
                                            LPOVERLAPPED    Overlapped);

/**
    @internal

    @brief
        Read, write or modify a register.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Type

        The type of register operation to perform.

    @param
        Size

        The size of the register being accessed.

    @param
        Block

        The register block that the register is located in.

    @param
        Offset

        The offset of the register within its register block.

    @param
        Data

        If reading the register, this is a pointer to a location that will be
        filled with the register's value. If writing or modifying the register,
        this is a pointer to a location holding the new register value.

    @param
        Mask

        If reading or writing the register, this value is ignored. If modifying
        the register, this is the mask denoting which bits should be modified.
        For example, if a bit in this value is '1' then that bit in the register
        will be modified to match the corresponding bit in the location pointed
        to by 'Data'.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_General_Internal_RegOp( aDIO_Handle             Handle,
                                        aDIO_Intrfc_Reg_Op_Type Type,
                                        aDIO_Intrfc_Reg_Op_Size Size,
                                        uint8                   Block,
                                        uint32                  Offset,
                                        uint32*                 Data,
                                        uint32                  Mask);

/**
    @internal

    @brief
        Controls locking of the 'Multi-Function' register to a particular mode.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Lock

        If TRUE, the 'Multi-Function' register will be locked, otherwise it
        will be unlocked.

    @param
        Mode

        If locking the 'Multi-Function' register, this is the mode to lock it
        to. Otherwise, this value is ignored.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_General_Internal_LockMultiFunction(
                                aDIO_Handle                             Handle,
                                BOOL                                    Lock,
                                aDIO_General_Internal_MultiFunc_Mode    Mode);

//============================================================================
//! @} End of General Internal Functions
//============================================================================

//============================================================================
//! @defgroup aDIO_General_Internal_Macro aDIOLib General Internal Macros
//! @{
//============================================================================

/**
    @brief
        Checks the validity of any pointer parameter.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is, any error message created will still be logged but
        won't be recorded in a device descriptor to be retrieved by the user.

    @param
        Param

        The pointer to check.

    @param
        Category

        The error category to use in case an error needs to be logged.

    @warning
        If the pointer is NULL (invalid), an appropriate error message will be
        logged and the function using this macro will immediately return
        ADIO_ERROR_CODE_BAD_PARAM.

*/
#define ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle, Param, Category) \
    if(Param == NULL)\
    {\
        return ADIO_LOG_INTERNAL_LOG_BAD_PARAM_PTR(Handle, Category, Param);\
    }

/**
    @brief
        Checks the validity of a aDIO_Handle parameter.

    @param
        Handle

        The handle to check. If this parameter is NULL, any error message
        created will still be logged but won't be recorded in a device
        descriptor to be retrieved by the user.

    @param
        Category

        The error category to use in case an error needs to be logged.

    @warning
        If the handle is NULL (invalid), an appropriate error message will be
        logged and the function using this macro will immediately return
        ADIO_ERROR_CODE_BAD_PARAM. If the handle is not NULL but the device
        file is an invalid handle, an appropriate error message will be
        logged and the function using this macro will immediately return
        ADIO_ERROR_CODE_UNEXPECTED.

*/
#define ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, Category) \
    if(Handle == NULL)\
    {\
        return ADIO_LOG_INTERNAL_LOG_BAD_PARAM_PTR(NULL, Category, Handle);\
    }\
    else if(Handle->dev_file == INVALID_HANDLE_VALUE)\
    {\
        return ADIO_LOG_INTERNAL_LOG_UNEXPECTED(\
                                NULL,\
                                Category,\
                                "The handle to the device file is invalid.");\
    }

//============================================================================
//! @} End of General Internal Macros
//============================================================================

//============================================================================
//! @} End of General Internal Module
//============================================================================

//============================================================================
//! @} End of Internal Module
//============================================================================


#ifdef __cplusplus
}
#endif

#endif // __ADIOLIB_GENERAL_INTERNAL_H__
