/**
    @file

    @brief
        This file implements the 'Interrupt' module of the aDIOLib API.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Interrupt.c 55136 2011-08-18 18:13:56Z jstrouse $
*/

#include "aDIOLib_Interrupt.h"
#include "aDIOLib_Interrupt_Internal.h"
#include "aDIOLib_General_Internal.h"
#include "aDIOLib_Log_Internal.h"


//============================================================================
// aDIOLib API
//============================================================================

//============================================================================
// aDIOLib Interrupt Module
//============================================================================

//============================================================================
// aDIOLib Interrupt Functions
//============================================================================

/*
    Configures interrupts and Wake-on-aDIO.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Interrupt_SetConfig( aDIO_Handle             Handle,
                                            BOOL                    IntEnable,
                                            aDIO_Interrupt_Int_Type Type,
                                            BOOL                    WakeEnable)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Interrupt_Internal_Module_State* int_state;
    aDIO_Intrfc_Int_Control int_control;
    aDIO_Intrfc_Wake_Control wake_control;
    int i;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_PTR(NULL,
                                    Handle,
                                    ADIO_ERROR_CATEGORY_INTERRUPT);
    if(IntEnable)
    {
        ADIO_INTERRUPT_INTERNAL_CHECK_INT_TYPE(Handle, Type);
    }

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    do
    {
        // Make a 'shortcut' to the Interrupt module state structure
        int_state = (aDIO_Interrupt_Internal_Module_State*)Handle->int_state;

        // Make sure the aDIO has interrupts
        if(Handle->dev_info.irq_number == 0)
        {
            result = ADIO_LOG_INTERNAL_LOG_NOT_CONFIG(
                                                Handle,
                                                ADIO_ERROR_CATEGORY_INTERRUPT,
                                                "An IRQ for this aDIO device");
            break;
        }

        // Handle interrupt configuration depending on if they're being enabled
        // or disabled
        if(IntEnable)
        {
            // Fill the IOCTL parameter struct
            int_control.type = aDIO_Interrupt_Internal_ConvertType(Type);
            int_control.change_state = TRUE;
            int_control.enable = IntEnable;

            // Call the interrupt control IOCTL
            result = aDIO_General_Internal_CallIoctl(
                                                Handle,
                                                IOCTL_ADIO_INT_CONTROL,
                                                &int_control,
                                                sizeof(aDIO_Intrfc_Int_Control),
                                                NULL);
            if(result != ADIO_ERROR_CODE_SUCCESS)
                break;
        }
        else
        {
            // Iterate through interrupt types and check if any are enabled
            for(i = 0; i < ADIO_INTERRUPT_INT_TYPE_COUNT; ++i)
            {
                int_control.type = aDIO_Interrupt_Internal_ConvertType(
                                                    (aDIO_Interrupt_Int_Type)i);
                int_control.change_state = FALSE;

                result = aDIO_General_Internal_CallIoctl(
                                                Handle,
                                                IOCTL_ADIO_INT_CONTROL,
                                                &int_control,
                                                sizeof(aDIO_Intrfc_Int_Control),
                                                NULL);
                if(result != ADIO_ERROR_CODE_SUCCESS)
                    break;

                // If an interrupt type is enabled, disable it and stop looping
                if(int_control.enable)
                {
                    // Fill the IOCTL parameter struct
                    int_control.change_state = TRUE;
                    int_control.enable = FALSE;

                    // Call the interrupt control IOCTL
                    result = aDIO_General_Internal_CallIoctl(
                                                Handle,
                                                IOCTL_ADIO_INT_CONTROL,
                                                &int_control,
                                                sizeof(aDIO_Intrfc_Int_Control),
                                                NULL);

                    break;
                }
            }
            if(result != ADIO_ERROR_CODE_SUCCESS)
                    break;
        }

        // If Wake-on-aDIO is supported, configure it
        if(Handle->dev_info.is_wake_supported)
        {
            // Fill the IOCTL parameter struct
            wake_control.change_enable = TRUE;
            wake_control.enable = WakeEnable;

            // Call the wake control IOCTL
            result = aDIO_General_Internal_CallIoctl(
                                            Handle,
                                            IOCTL_ADIO_WAKE_CONTROL,
                                            &wake_control,
                                            sizeof(aDIO_Intrfc_Wake_Control),
                                            NULL);
            if(result != ADIO_ERROR_CODE_SUCCESS)
                break;
        }

    } while(0);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Gets the current configuration of interrupts and Wake-on-aDIO.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Interrupt_GetConfig(
                                        aDIO_Handle                 Handle,
                                        BOOL*                       IntEnable,
                                        aDIO_Interrupt_Int_Type*    Type,
                                        BOOL*                       WakeEnable)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Interrupt_Internal_Module_State* int_state;
    aDIO_Intrfc_Int_Control int_control;
    aDIO_Intrfc_Wake_Control wake_control;
    int i;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_PTR(NULL,
                                    Handle,
                                    ADIO_ERROR_CATEGORY_INTERRUPT);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle,
                                    IntEnable,
                                    ADIO_ERROR_CATEGORY_INTERRUPT);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle,
                                    Type,
                                    ADIO_ERROR_CATEGORY_INTERRUPT);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    do
    {
        // Make a 'shortcut' to the Interrupt module state structure
        int_state = (aDIO_Interrupt_Internal_Module_State*)Handle->int_state;

        // Make sure the aDIO has interrupts
        if(Handle->dev_info.irq_number == 0)
        {
            result = ADIO_LOG_INTERNAL_LOG_NOT_CONFIG(
                                                Handle,
                                                ADIO_ERROR_CATEGORY_INTERRUPT,
                                                "An IRQ for this aDIO device");
            break;
        }

        // Assume interrupts are disabled
        *IntEnable = FALSE;
        *Type = ADIO_INTERRUPT_INT_TYPE_COUNT;

        // Iterate through interrupt types and check if any are enabled
        for(i = 0; i < ADIO_INTERRUPT_INT_TYPE_COUNT; ++i)
        {
            int_control.type = aDIO_Interrupt_Internal_ConvertType(
                                                    (aDIO_Interrupt_Int_Type)i);
            int_control.change_state = FALSE;

            result = aDIO_General_Internal_CallIoctl(
                                                Handle,
                                                IOCTL_ADIO_INT_CONTROL,
                                                &int_control,
                                                sizeof(aDIO_Intrfc_Int_Control),
                                                NULL);
            if(result != ADIO_ERROR_CODE_SUCCESS)
                break;

            if(int_control.enable)
            {
                *IntEnable = TRUE;
                *Type = (aDIO_Interrupt_Int_Type)i;
                break;
            }
        }
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // If Wake-on-aDIO configuration is requested, retrieve it
        if(WakeEnable != NULL)
        {
            if(!Handle->dev_info.is_wake_supported)
            {
                *WakeEnable = FALSE;
            }
            else
            {
                // Fill the IOCTL parameter struct
                wake_control.change_enable = FALSE;

                // Call the wake control IOCTL
                result = aDIO_General_Internal_CallIoctl(
                                            Handle,
                                            IOCTL_ADIO_WAKE_CONTROL,
                                            &wake_control,
                                            sizeof(aDIO_Intrfc_Wake_Control),
                                            NULL);
                if(result != ADIO_ERROR_CODE_SUCCESS)
                    break;

                // Get the Wake-on-aDIO state
                *WakeEnable = wake_control.enable;
            }
        }

    } while(0);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Retrieves the interrupt count for a particular interrupt type.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Interrupt_Poll(  aDIO_Handle             Handle,
                                        aDIO_Interrupt_Int_Type Type,
                                        uint32*                 Count,
                                        uint8*                  CompareValue,
                                        uint8*                  Port0Value,
                                        uint8*                  Port1Value)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Interrupt_Internal_Module_State* int_state;
    aDIO_Intrfc_Int_Status int_status;
    aDIO_Interrupt_Type driver_type;

    // Check the parameters (except the compare and port values because they
    // can be NULL)
    ADIO_GENERAL_INTERNAL_CHECK_PTR(NULL,
                                    Handle,
                                    ADIO_ERROR_CATEGORY_INTERRUPT);
    ADIO_INTERRUPT_INTERNAL_CHECK_INT_TYPE(Handle, Type);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle,
                                    Count,
                                    ADIO_ERROR_CATEGORY_INTERRUPT);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    do
    {
        // Make a 'shortcut' to the Interrupt module state structure
        int_state = (aDIO_Interrupt_Internal_Module_State*)Handle->int_state;

        // Make sure the aDIO has interrupts
        if(Handle->dev_info.irq_number == 0)
        {
            result = ADIO_LOG_INTERNAL_LOG_NOT_CONFIG(
                                                Handle,
                                                ADIO_ERROR_CATEGORY_INTERRUPT,
                                                "An IRQ for this aDIO device");
            break;
        }

        // Convert the library interrupt type into a driver interrupt type
        driver_type = aDIO_Interrupt_Internal_ConvertType(Type);

        // Check module state
        if(int_state->types[driver_type].is_callback_installed)
        {
            result = ADIO_LOG_INTERNAL_LOG_IN_USE(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT,
                                        "An interrupt callback for this type");
            break;
        }

        // Fill the IOCTL parameter struct
        int_status.type = driver_type;
        int_status.op = ADIO_INTRFC_INT_STATUS_OP_POLL;

        // Call the interrupt status IOCTL
        result = aDIO_General_Internal_CallIoctl(
                                                Handle,
                                                IOCTL_ADIO_INT_STATUS,
                                                &int_status,
                                                sizeof(aDIO_Intrfc_Int_Status),
                                                NULL);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Get the interrupt status values out
        *Count = int_status.count;
        if(CompareValue != NULL)
            *CompareValue = int_status.last_compare_val;
        if(Port0Value != NULL)
            *Port0Value = int_status.last_port0_val;
        if(Port1Value != NULL)
            *Port1Value = int_status.last_port1_val;

    } while(0);


    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Installs a callback function for a particular interrupt type.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Interrupt_InstallCallback(
                                            aDIO_Handle             Handle,
                                            aDIO_Interrupt_Int_Type Type,
                                            aDIO_Interrupt_Callback Callback,
                                            void*                   UserPtr)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Interrupt_Internal_Module_State* int_state;
    aDIO_Interrupt_Type driver_type;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_PTR(NULL,
                                    Handle,
                                    ADIO_ERROR_CATEGORY_INTERRUPT);
    ADIO_INTERRUPT_INTERNAL_CHECK_INT_TYPE(Handle, Type);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle,
                                    Callback,
                                    ADIO_ERROR_CATEGORY_INTERRUPT);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    do
    {
        // Make a 'shortcut' to the Interrupt module state structure
        int_state = (aDIO_Interrupt_Internal_Module_State*)Handle->int_state;

        // Make sure the aDIO has interrupts
        if(Handle->dev_info.irq_number == 0)
        {
            result = ADIO_LOG_INTERNAL_LOG_NOT_CONFIG(
                                                Handle,
                                                ADIO_ERROR_CATEGORY_INTERRUPT,
                                                "An IRQ for this aDIO device");
            break;
        }

        // Convert the library interrupt type into a driver interrupt type
        driver_type = aDIO_Interrupt_Internal_ConvertType(Type);

        // Check module state
        if(int_state->types[driver_type].is_callback_installed)
        {
            result = ADIO_LOG_INTERNAL_LOG_IN_USE(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT,
                                        "An interrupt callback for this type");
            break;
        }

        // Do the callback installation
        result = aDIO_Interrupt_Internal_Callback_Install(  Handle,
                                                            Type,
                                                            Callback,
                                                            UserPtr);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

    } while(0);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Removes a callback function for a particular interrupt type.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Interrupt_RemoveCallback(aDIO_Handle             Handle,
                                                aDIO_Interrupt_Int_Type Type)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Interrupt_Internal_Module_State* int_state;
    aDIO_Interrupt_Type driver_type;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_PTR(NULL,
                                    Handle,
                                    ADIO_ERROR_CATEGORY_INTERRUPT);
    ADIO_INTERRUPT_INTERNAL_CHECK_INT_TYPE(Handle, Type);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    do
    {
        // Make a 'shortcut' to the Interrupt module state structure
        int_state = (aDIO_Interrupt_Internal_Module_State*)Handle->int_state;

        // Make sure the aDIO has interrupts
        if(Handle->dev_info.irq_number == 0)
        {
            result = ADIO_LOG_INTERNAL_LOG_NOT_CONFIG(
                                                Handle,
                                                ADIO_ERROR_CATEGORY_INTERRUPT,
                                                "An IRQ for this aDIO device");
            break;
        }

        // Convert the library interrupt type into a driver interrupt type
        driver_type = aDIO_Interrupt_Internal_ConvertType(Type);

        // Check module state
        if(!int_state->types[driver_type].is_callback_installed)
        {
            // If this type doesn't have a callback installed, just return
            //  success
            break;
        }

        // Do the callback removal
        result = aDIO_Interrupt_Internal_Callback_Remove(Handle, Type);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

    } while(0);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

//============================================================================
// End of Interrupt Functions
//============================================================================

//============================================================================
// End of Interrupt Module
//============================================================================

//============================================================================
// End of API
//============================================================================

