/**
    @file

    @brief
        This file implements internal functions for the 'Interrupt' module of
        the aDIOLib.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Interrupt_Internal.c 55143 2011-08-18 19:24:22Z jstrouse $
*/

#include "aDIOLib_Interrupt_Internal.h"
#include "aDIOLib_General_Internal.h"
#include "aDIOLib_Log_Internal.h"


//============================================================================
// aDIOLib Internal Module
//============================================================================

//============================================================================
// aDIOLib Interrupt Internal Module
//============================================================================

//============================================================================
// aDIOLib Interrupt Internal Functions
//============================================================================

//============================================================================
// aDIOLib Interrupt Callback Functions
//============================================================================

/*
    Performs the work of installing an interrupt callback function.
*/
aDIO_Error aDIO_Interrupt_Internal_Callback_Install(
                                            aDIO_Handle             Handle,
                                            aDIO_Interrupt_Int_Type Type,
                                            aDIO_Interrupt_Callback Callback,
                                            void*                   UserPtr)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Interrupt_Internal_Module_State* int_state;
    aDIO_Interrupt_Internal_Thread_Data* thread_data;
    aDIO_Interrupt_Type driver_type;
    HANDLE thread_handle = NULL;
    DWORD thread_id;
    BOOL force_terminate = FALSE;

    // Make a 'shortcut' to the Interrupt module state structure
    int_state = (aDIO_Interrupt_Internal_Module_State*)Handle->int_state;

    // Convert the library interrupt type into a driver interrupt type
    driver_type = aDIO_Interrupt_Internal_ConvertType(Type);

    // Allocate the resources needed by the callback thread
    result = aDIO_Interrupt_Internal_Callback_AllocateResources(Handle,
                                                                Type,
                                                                Callback,
                                                                UserPtr,
                                                                &thread_data);
    if(result != ADIO_ERROR_CODE_SUCCESS)
        return result;

    do
    {
        // Create the callback thread
        thread_handle = CreateThread(
                                    NULL,
                                    0,
                                    aDIO_Interrupt_Internal_Callback_ThreadFunc,
                                    (void*)thread_data,
                                    0,
                                    &thread_id);
        if(thread_handle == NULL)
        {
            result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "CreateThread");
            break;
        }

        // Wait for the callback thread to be ready or report an error
        result = aDIO_Interrupt_Internal_Callback_WaitThreadState(
                                Handle,
                                thread_data,
                                ADIO_INTERRUPT_INTERNAL_THREAD_STATE_RUNNING,
                                &force_terminate);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Callback installation is complete. Record the thread handle and the
        //  pointer to the thread data in the interrupt module state. Mark that
        //  the callback has been installed.
        int_state->types[driver_type].thread_handle = thread_handle;
        int_state->types[driver_type].thread_data = thread_data;
        int_state->types[driver_type].is_callback_installed = TRUE;

    } while(0);

    // If there was an error then cleanup
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        // If the thread was created but encountered an error, try to finish its
        //  shutdown before doing cleanup
        if(thread_handle && !force_terminate)
        {
            // Set the 'removing' state
            InterlockedExchange(&thread_data->state,
                                ADIO_INTERRUPT_INTERNAL_THREAD_STATE_REMOVING);

            // Wait for the 'complete' state or for an error
            aDIO_Interrupt_Internal_Callback_WaitThreadState(
                                Handle,
                                thread_data,
                                ADIO_INTERRUPT_INTERNAL_THREAD_STATE_COMPLETE,
                                &force_terminate);
        }

        aDIO_Interrupt_Internal_Callback_CleanupResources(  Handle,
                                                            thread_handle,
                                                            thread_data,
                                                            force_terminate);
    }

    return result;
}

/*
    Allocates and prepares all resources needed by the interrupt callback
    thread.
*/
aDIO_Error aDIO_Interrupt_Internal_Callback_AllocateResources(
                            aDIO_Handle                             Handle,
                            aDIO_Interrupt_Int_Type                 Type,
                            aDIO_Interrupt_Callback                 Callback,
                            void*                                   UserPtr,
                            aDIO_Interrupt_Internal_Thread_Data**   ThreadData)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Interrupt_Internal_Thread_Data* thread_data;
    aDIO_Interrupt_Type driver_type;
    uint32 i;

    // Convert the library interrupt type into a driver interrupt type
    driver_type = aDIO_Interrupt_Internal_ConvertType(Type);

    // Allocate space for the thread data structure
    thread_data = (aDIO_Interrupt_Internal_Thread_Data*)malloc(
                                sizeof(aDIO_Interrupt_Internal_Thread_Data));
    if(!thread_data)
    {
        *ThreadData = NULL;
        return ADIO_LOG_INTERNAL_LOG_NO_MEMORY(
                                Handle,
                                ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                sizeof(aDIO_Interrupt_Internal_Thread_Data));
    }

    // Zero the thread data struct
    memset(thread_data, 0, sizeof(aDIO_Interrupt_Internal_Thread_Data));

    // Fill the thread data struct
    thread_data->handle = Handle;
    thread_data->type = Type;
    thread_data->driver_type = driver_type;
    thread_data->terminate_event = NULL;
    thread_data->state = ADIO_INTERRUPT_INTERNAL_THREAD_STATE_INSTALLING;
    thread_data->error_code = ADIO_ERROR_CODE_SUCCESS;
    thread_data->callback = Callback;
    thread_data->user_pointer = UserPtr;

    // Setup the interrupt status ioctl structures
    for(i = 0; i < ADIO_INTERRUPT_INTERNAL_QUEUE_DEPTH; ++i)
    {
        thread_data->int_status[i].type = driver_type;
        thread_data->int_status[i].op = ADIO_INTRFC_INT_STATUS_OP_NOTIFY;
    }

    do
    {
        // Create the event used to terminate the thread
        thread_data->terminate_event = CreateEvent(NULL, TRUE, FALSE, "");
        if(thread_data->terminate_event == NULL)
        {
            result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "CreateEvent");
            break;
        }

        // Create the notification events
        for(i = 0; i < ADIO_INTERRUPT_INTERNAL_QUEUE_DEPTH; ++i)
        {
            // Use manual-reset events that start out signaled to make clean up
            //  easier
            thread_data->overlappeds[i].hEvent = CreateEvent(   NULL,
                                                                TRUE,
                                                                TRUE,
                                                                "");
            if(!thread_data->overlappeds[i].hEvent)
            {
                result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "CreateEvent");
                break;
            }
        }
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Queue the notification events
        for(i = 0; i < ADIO_INTERRUPT_INTERNAL_QUEUE_DEPTH; ++i)
        {
            // Reset the notification event before queuing it
            if(!ResetEvent(thread_data->overlappeds[i].hEvent))
            {
                result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "ResetEvent");
                break;
            }

            // Add the event to the notification queue
            result = aDIO_General_Internal_CallIoctl(
                                                Handle,
                                                IOCTL_ADIO_INT_STATUS,
                                                &thread_data->int_status[i],
                                                sizeof(aDIO_Intrfc_Int_Status),
                                                &thread_data->overlappeds[i]);
            if(result != ADIO_ERROR_CODE_SUCCESS)
            {
                // Set the event since it wasn't actually queued
                if(!SetEvent(thread_data->overlappeds[i].hEvent))
                {
                    ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "SetEvent");
                }

                break;
            }
        }
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Set the thread data pointer
        *ThreadData = thread_data;

    } while(0);

    // If there was an error, cleanup before returning
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        aDIO_Interrupt_Internal_Callback_CleanupResources(  Handle,
                                                            NULL,
                                                            thread_data,
                                                            FALSE);
    }

    return result;
}

/*
    Waits, with a timeout, for the given thread to reach a specific thread state
    or to report an error.
*/
aDIO_Error aDIO_Interrupt_Internal_Callback_WaitThreadState(
                            aDIO_Handle                             Handle,
                            aDIO_Interrupt_Internal_Thread_Data*    ThreadData,
                            aDIO_Interrupt_Internal_Thread_State    GoalState,
                            BOOL*                                   ForceTerm)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Interrupt_Internal_Thread_State thread_state;
    int32 attempts = 0;

    do
    {
        // Record that another attempt is being made
        attempts++;

        // If the attempt limit has been reached, log an error and return
        if(attempts > ADIO_INTERRUPT_INTERNAL_THREAD_STATE_WAIT_LIMIT)
        {
            result = ADIO_LOG_INTERNAL_LOG_ANY(
                                    Handle,
                                    ADIO_ERROR_CODE_UNEXPECTED,
                                    ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                    ADIO_ERROR_SEVERITY_ERROR,
                                    "Waiting for thread state %d timed out. "\
                                        "Final thread state was %d.",
                                    GoalState,
                                    thread_state);

            break;
        }

        // Give the thread a chance to run
        Sleep(ADIO_INTERRUPT_INTERNAL_THREAD_STATE_WAIT_TIME);

        // Get the thread's current state
        thread_state = InterlockedExchangeAdd(&ThreadData->state, 0);

        // If the thread encountered an error, return the error code
        if(thread_state == ADIO_INTERRUPT_INTERNAL_THREAD_STATE_ERROR)
        {
            result = ThreadData->error_code;
            break;
        }

    } while(thread_state != GoalState);

    // Determine if the thread needs to be forcibly terminated, based on the
    //  thread state
    if( thread_state == ADIO_INTERRUPT_INTERNAL_THREAD_STATE_RUNNING ||
        thread_state == ADIO_INTERRUPT_INTERNAL_THREAD_STATE_COMPLETE ||
        thread_state == ADIO_INTERRUPT_INTERNAL_THREAD_STATE_ERROR)
    {
        *ForceTerm = FALSE;
    }
    else
    {
        *ForceTerm = TRUE;
    }

    return result;
}

/*
    Spins until the 'global lock' is obtained, the thread's termination event is
    set, or until an error occurs.
*/
aDIO_Error aDIO_Interrupt_Internal_Callback_ObtainGlobalLock(
                            aDIO_Interrupt_Internal_Thread_Data*    ThreadData,
                            BOOL*                                   StopThread)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    DWORD wait_result;

    // Assume the lock will be obtained
    *StopThread = FALSE;

    // Try to obtain the global lock
    while(!TryEnterCriticalSection(&ThreadData->handle->global_lock))
    {
        // Global lock is already locked, check if thread has been
        //  terminated
        wait_result = WaitForSingleObject(ThreadData->terminate_event, 0);
        if(wait_result == WAIT_OBJECT_0)
        {
            // The thread was terminated
            *StopThread = TRUE;
            break;
        }
        else if(wait_result == WAIT_TIMEOUT)
        {
            // Give the global lock a chance to be released
            Sleep(0);
        }
        else
        {
            // An error occurred
            result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        ThreadData->handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "WaitForSingleObject");
            *StopThread = TRUE;
            break;
        }
    }

    return result;
}

/*
    Handles getting notification results and then re-queuing the notification.
*/
aDIO_Error aDIO_Interrupt_Internal_Callback_HandleNotify(
                        aDIO_Interrupt_Internal_Thread_Data*    ThreadData,
                        aDIO_Interrupt_Callback_Info*           CallbackInfo,
                        uint32                                  CurrentEvent)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    DWORD num;

    // Get the complete result of the interrupt notification
    if(!GetOverlappedResult(ThreadData->handle->dev_file,
                            &ThreadData->overlappeds[CurrentEvent],
                            &num,
                            FALSE))
    {
        return ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        ThreadData->handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "GetOverlappedResult");
    }

    // Set the info for the callback
    CallbackInfo->interrupt_count = ThreadData->int_status[CurrentEvent].count;
    CallbackInfo->compare_val =
                        ThreadData->int_status[CurrentEvent].last_compare_val;
    CallbackInfo->port0_val =
                        ThreadData->int_status[CurrentEvent].last_port0_val;
    CallbackInfo->port1_val =
                        ThreadData->int_status[CurrentEvent].last_port1_val;

    // Reset the notification event before re-queuing it
    if(!ResetEvent(ThreadData->overlappeds[CurrentEvent].hEvent))
    {
        return ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        ThreadData->handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "ResetEvent");
    }

    // Re-queue the notification event
    result = aDIO_General_Internal_CallIoctl(
                                        ThreadData->handle,
                                        IOCTL_ADIO_INT_STATUS,
                                        &ThreadData->int_status[CurrentEvent],
                                        sizeof(aDIO_Intrfc_Int_Status),
                                        &ThreadData->overlappeds[CurrentEvent]);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        // Set the event since it wasn't actually queued
        if(!SetEvent(ThreadData->overlappeds[CurrentEvent].hEvent))
        {
            ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        ThreadData->handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "SetEvent");
        }
    }

    return result;
}

/*
    Performs the core operations of the callback thread while in its 'running'
    state.
*/
aDIO_Error aDIO_Interrupt_Internal_Callback_ThreadCore(
                            aDIO_Interrupt_Internal_Thread_Data* ThreadData)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Interrupt_Callback_Info callback_info;
    HANDLE wait_events[2];
    DWORD wait_result;
    uint32 current_event = 0;
    BOOL stop_thread;

    // Load the events being waited on
    wait_events[0] = ThreadData->terminate_event;
    wait_events[1] = ThreadData->overlappeds[0].hEvent;

    // Setup the callback info structure
    callback_info.type = ThreadData->type;
    callback_info.result = ADIO_ERROR_CODE_SUCCESS;
    callback_info.user_pointer = ThreadData->user_pointer;

    // Start the main 'working' loop
    while(TRUE)
    {
        // Wait for an event to fire
        wait_result = WaitForMultipleObjects(2, wait_events, FALSE, INFINITE);

        if(wait_result == WAIT_OBJECT_0)
        {
            // The termination event was signaled
            break;
        }
        else if(wait_result != (WAIT_OBJECT_0 + 1))
        {
            // An error occurred: log it, call the user callback and exit the
            //  thread loop
            result = ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        ThreadData->handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "WaitForMultipleObjects");

            callback_info.result = result;
            (ThreadData->callback)(callback_info);

            break;
        }

        // The notification event was set

        // Try to obtain the global lock
        result = aDIO_Interrupt_Internal_Callback_ObtainGlobalLock(
                                                                ThreadData,
                                                                &stop_thread);
        if(stop_thread)
        {
            // An error has occurred or the thread's termination event has been
            //  set. If an error has occurred, call the user's callback before
            //  exiting the thread loop
            if(result != ADIO_ERROR_CODE_SUCCESS)
            {
                callback_info.result = result;
                (ThreadData->callback)(callback_info);
            }
            break;
        }

        // Handle the notification
        result = aDIO_Interrupt_Internal_Callback_HandleNotify( ThreadData,
                                                                &callback_info,
                                                                current_event);

        if(result == ADIO_ERROR_CODE_SUCCESS)
        {
            // Increment the current event index
            current_event++;
            current_event %= ADIO_INTERRUPT_INTERNAL_QUEUE_DEPTH;

            // Update the event to wait on next
            wait_events[1] = ThreadData->overlappeds[current_event].hEvent;
        }

        // Release the global lock
        LeaveCriticalSection(&ThreadData->handle->global_lock);

        // Call the user's callback for normal notification, or if there was an
        //  error
        callback_info.result = result;
        (ThreadData->callback)(callback_info);

        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;
    }

    return result;
}

/*
    Created in a new thread to monitor a specific interrupt and notify the user
    when it occurs.
*/
DWORD WINAPI aDIO_Interrupt_Internal_Callback_ThreadFunc(void* ThreadData)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Interrupt_Internal_Thread_Data* thread_data;
    aDIO_Interrupt_Internal_Thread_State thread_state;

    // Setup the pointer to the callback thread data
    thread_data = (aDIO_Interrupt_Internal_Thread_Data*)ThreadData;

    // Set the thread state to 'running'
    InterlockedExchange(&thread_data->state,
                        ADIO_INTERRUPT_INTERNAL_THREAD_STATE_RUNNING);

    // Start the main thread loop
    result = aDIO_Interrupt_Internal_Callback_ThreadCore(thread_data);
    if(result != ADIO_ERROR_CODE_SUCCESS)
    {
        // The run-time error has already been reported to the user's callback
        //  so set the 'error' state and wait for the 'removing' state
        InterlockedExchange(&thread_data->state,
                            ADIO_INTERRUPT_INTERNAL_THREAD_STATE_ERROR);
    }

    // Wait for the 'removing' state to be set
    do
    {
        // Allow the 'main' thread to set the 'removing state'
        Sleep(0);

        // Get the current thread state
        thread_state = InterlockedExchangeAdd(&thread_data->state, 0);

    } while(thread_state != ADIO_INTERRUPT_INTERNAL_THREAD_STATE_REMOVING);

    // Clear the interrupt status notification queue
    result = aDIO_Interrupt_Internal_Callback_ClearNotificationQueue(
                                                            thread_data->handle,
                                                            thread_data);

    // Set the final thread state
    if(result == ADIO_ERROR_CODE_SUCCESS)
    {
        InterlockedExchange(&thread_data->state,
                            ADIO_INTERRUPT_INTERNAL_THREAD_STATE_COMPLETE);
    }
    else
    {
        thread_data->error_code = result;
        InterlockedExchange(&thread_data->state,
                            ADIO_INTERRUPT_INTERNAL_THREAD_STATE_ERROR);
    }

    return 0;
}

/*
    Clears all interrupt notification events from a callback thread's
    notification queue.
*/
aDIO_Error aDIO_Interrupt_Internal_Callback_ClearNotificationQueue(
                            aDIO_Handle                             Handle,
                            aDIO_Interrupt_Internal_Thread_Data*    ThreadData)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Intrfc_Int_Status int_status;
    DWORD wait_result;
    uint8 i;

    // Clear the interrupt's notification queue
    int_status.type = ThreadData->driver_type;
    int_status.op = ADIO_INTRFC_INT_STATUS_OP_CLEAR_QUEUE;
    result = aDIO_General_Internal_CallIoctl(   Handle,
                                                IOCTL_ADIO_INT_STATUS,
                                                &int_status,
                                                sizeof(aDIO_Intrfc_Int_Status),
                                                NULL);
    if(result != ADIO_ERROR_CODE_SUCCESS)
        return result;

    // Wait for all (existing) notification IOCTLs to be canceled
    for(i = 0; i < ADIO_INTERRUPT_INTERNAL_QUEUE_DEPTH; ++i)
    {
        if(ThreadData->overlappeds[i].hEvent != NULL)
        {
            wait_result = WaitForSingleObject(
                                    ThreadData->overlappeds[i].hEvent,
                                    ADIO_INTERRUPT_INTERNAL_EVENT_WAIT_TIME);
            if(wait_result == WAIT_TIMEOUT)
            {
                ADIO_LOG_INTERNAL_LOG_UNEXPECTED(
                                    Handle,
                                    ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                    "Timed out while waiting for an interrupt "\
                                        "notification cancellation.");
                if(result == ADIO_ERROR_CODE_SUCCESS)
                    result = ADIO_ERROR_CODE_UNEXPECTED;
            }
            else if(wait_result != WAIT_OBJECT_0)
            {
                ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "WaitForSingleObject");
                if(result == ADIO_ERROR_CODE_SUCCESS)
                    result = ADIO_ERROR_CODE_WIN_API;
            }
        }
    }

    return result;
}

/*
    Cleans up thread-related resources.
*/
aDIO_Error aDIO_Interrupt_Internal_Callback_CleanupResources(
                        aDIO_Handle                             Handle,
                        HANDLE                                  ThreadHandle,
                        aDIO_Interrupt_Internal_Thread_Data*    ThreadData,
                        BOOL                                    ForceTerm)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result;
    DWORD exit_code;
    int32 attempts = 0;
    uint8 i;

    // If the thread data has not been allocated there is nothing to clean up
    if(!ThreadData)
        return result;

    if(ThreadHandle)
    {
        if(ForceTerm)
        {
            // Clear the interrupt status notification queue
            temp_result =
                    aDIO_Interrupt_Internal_Callback_ClearNotificationQueue(
                                                                    Handle,
                                                                    ThreadData);
            if(result == ADIO_ERROR_CODE_SUCCESS)
                result = temp_result;

            // Forcibly terminate the thread
            if(!TerminateThread(ThreadHandle, 0))
            {
                ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "TerminateThread");

                if(result == ADIO_ERROR_CODE_SUCCESS)
                    result = ADIO_ERROR_CODE_WIN_API;
            }

            // Wait (with a limit) for the thread to exit
            do
            {
                attempts++;
                if(attempts > ADIO_INTERRUPT_INTERNAL_FORCE_TERM_WAIT_LIMIT)
                    break;

                if(!GetExitCodeThread(ThreadHandle, &exit_code))
                {
                    ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "GetExitCodeThread");

                    if(result == ADIO_ERROR_CODE_SUCCESS)
                        result = ADIO_ERROR_CODE_WIN_API;

                    break;
                }

                // Give the thread a chance to close
                Sleep(ADIO_INTERRUPT_INTERNAL_FORCE_TERM_WAIT_TIME);

            } while(exit_code == STILL_ACTIVE);
        }

        // Close the thread handle
        if(!CloseHandle(ThreadHandle))
        {
            ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "CloseHandle");

            if(result == ADIO_ERROR_CODE_SUCCESS)
                result = ADIO_ERROR_CODE_WIN_API;
        }
    }

    // Close all events created for notification
    for(i = 0; i < ADIO_INTERRUPT_INTERNAL_QUEUE_DEPTH; ++i)
    {
        if(ThreadData->overlappeds[i].hEvent)
        {
            if(!CloseHandle(ThreadData->overlappeds[i].hEvent))
            {
                ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "CloseHandle");
                if(result == ADIO_ERROR_CODE_SUCCESS)
                    result = ADIO_ERROR_CODE_WIN_API;
            }
        }
    }

    // Close the handle to the termination event
    if(ThreadData->terminate_event)
    {
        if(!CloseHandle(ThreadData->terminate_event))
        {
            ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "CloseHandle");

            if(result == ADIO_ERROR_CODE_SUCCESS)
                result = ADIO_ERROR_CODE_WIN_API;
        }
    }

    // Free the memory for the thread data structure
    free(ThreadData);

    return result;
}

/*
    Performs the work of removing an interrupt callback function.
*/
aDIO_Error aDIO_Interrupt_Internal_Callback_Remove(
                                            aDIO_Handle             Handle,
                                            aDIO_Interrupt_Int_Type Type)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result;
    aDIO_Interrupt_Internal_Module_State* int_state;
    aDIO_Interrupt_Internal_Thread_Data* thread_data;
    aDIO_Interrupt_Internal_Thread_State thread_state;
    aDIO_Interrupt_Type driver_type;
    HANDLE thread_handle;
    BOOL force_terminate;

    // Make a 'shortcut' to the Interrupt module state structure
    int_state = (aDIO_Interrupt_Internal_Module_State*)Handle->int_state;

    // Convert the library interrupt type into a driver interrupt type
    driver_type = aDIO_Interrupt_Internal_ConvertType(Type);

    // Return success if there is no callback installed
    if(!int_state->types[driver_type].is_callback_installed)
        return ADIO_ERROR_CODE_SUCCESS;

    // 'Detach' the thread data structure and thread handle from the interrupt
    //  module state in the device descriptor
    thread_handle = int_state->types[driver_type].thread_handle;
    int_state->types[driver_type].thread_handle = NULL;

    thread_data = int_state->types[driver_type].thread_data;
    int_state->types[driver_type].thread_data = NULL;

    // Get the current thread state
    thread_state = InterlockedExchangeAdd(&thread_data->state, 0);

    if( thread_state != ADIO_INTERRUPT_INTERNAL_THREAD_STATE_RUNNING &&
        thread_state != ADIO_INTERRUPT_INTERNAL_THREAD_STATE_ERROR)
    {
        // Unknown thread state: log an error and try to shut down the thread
        //  anyway
        force_terminate = TRUE;

        ADIO_LOG_INTERNAL_LOG_ANY(  Handle,
                                    ADIO_ERROR_CODE_UNEXPECTED,
                                    ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                    ADIO_ERROR_SEVERITY_ERROR,
                                    "The interrupt callback for type %d"\
                                        " had unexpected state %d.",
                                    Type,
                                    thread_state);

        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = ADIO_ERROR_CODE_UNEXPECTED;
    }
    else
    {
        force_terminate = FALSE;

        // If the thread is running, stop it by setting the termination event
        if(thread_state == ADIO_INTERRUPT_INTERNAL_THREAD_STATE_RUNNING)
        {
            if(!SetEvent(thread_data->terminate_event))
            {
                ADIO_LOG_INTERNAL_LOG_WIN_API(
                                        Handle,
                                        ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                        "SetEvent");

                if(result == ADIO_ERROR_CODE_SUCCESS)
                    result = ADIO_ERROR_CODE_WIN_API;
            }
        }

        // Set the thread's state to 'removing' to allow it to shut down
        InterlockedExchange(&thread_data->state,
                            ADIO_INTERRUPT_INTERNAL_THREAD_STATE_REMOVING);

        // Wait for the thread to shut down or report an error
        temp_result = aDIO_Interrupt_Internal_Callback_WaitThreadState(
                                Handle,
                                thread_data,
                                ADIO_INTERRUPT_INTERNAL_THREAD_STATE_COMPLETE,
                                &force_terminate);
        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = temp_result;
    }

    // Cleanup all thread resources (and forcibly stop the thread if needed)
    temp_result = aDIO_Interrupt_Internal_Callback_CleanupResources(
                                                            Handle,
                                                            thread_handle,
                                                            thread_data,
                                                            force_terminate);
    if(result == ADIO_ERROR_CODE_SUCCESS)
        result = temp_result;

    // Mark that the callback was removed, even if an error occurred
    int_state->types[driver_type].is_callback_installed = FALSE;

    return result;
}

//============================================================================
// End of Interrupt Callback Functions
//============================================================================

/*
    Takes a library interrupt type and converts it into a driver interrupt type.
*/
aDIO_Interrupt_Type aDIO_Interrupt_Internal_ConvertType(
                                                aDIO_Interrupt_Int_Type Type)
{
    // Convert between types
    switch(Type)
    {
    case ADIO_INTERRUPT_INT_TYPE_STROBE:
        return ADIO_INT_TYPE_STROBE;
        break;
    case ADIO_INTERRUPT_INT_TYPE_EVENT:
        return ADIO_INT_TYPE_EVENT;
        break;
    case ADIO_INTERRUPT_INT_TYPE_MATCH:
        return ADIO_INT_TYPE_MATCH;
        break;
    }

    return ADIO_INT_TYPE_COUNT;
}

/*
    Prepares the Interrupt module for operation.
*/
aDIO_Error aDIO_Interrupt_Internal_PrepareModule(aDIO_Handle Handle)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;

    do
    {
        // Allocate the space needed for the Interrupt state structure
        Handle->int_state = malloc(
                                sizeof(aDIO_Interrupt_Internal_Module_State));
        if(!Handle->int_state)
        {
            result = ADIO_LOG_INTERNAL_LOG_NO_MEMORY(
                                Handle,
                                ADIO_ERROR_CATEGORY_INTERRUPT_INTERNAL,
                                sizeof(aDIO_Interrupt_Internal_Module_State));
            break;
        }

        // Zero the Interrupt module state structure
        memset( Handle->int_state,
                0,
                sizeof(aDIO_Interrupt_Internal_Module_State));

    } while(0);

    return result;
}

/*
    Resets the Interrupt module back to its default state.
*/
aDIO_Error aDIO_Interrupt_Internal_ResetModule(aDIO_Handle Handle)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result;
    aDIO_Interrupt_Internal_Module_State* int_state;
    aDIO_Intrfc_Int_Control int_control;
    uint8 i;

    // Make a 'shortcut' to the Interrupt module state structure
    int_state = (aDIO_Interrupt_Internal_Module_State*)Handle->int_state;

    // Make sure the aDIO has interrupts
    if(Handle->dev_info.irq_number != 0)
    {
        // Remove all interrupt callbacks
        for(i = 0; i < ADIO_INTERRUPT_INT_TYPE_COUNT; ++i)
        {
            temp_result = aDIO_Interrupt_Internal_Callback_Remove(
                                                    Handle,
                                                    (aDIO_Interrupt_Int_Type)i);
            if(result == ADIO_ERROR_CODE_SUCCESS)
                result = temp_result;
        }

        // Disable all interrupts
        for(i = 0; i < ADIO_INTERRUPT_INT_TYPE_COUNT; ++i)
        {
            int_control.type = aDIO_Interrupt_Internal_ConvertType(
                                                    (aDIO_Interrupt_Int_Type)i);
            int_control.change_state = TRUE;
            int_control.enable = FALSE;

            temp_result = aDIO_General_Internal_CallIoctl(
                                                Handle,
                                                IOCTL_ADIO_INT_CONTROL,
                                                &int_control,
                                                sizeof(aDIO_Intrfc_Int_Control),
                                                NULL);
            if(result == ADIO_ERROR_CODE_SUCCESS)
                result = temp_result;
        }
    }

    return result;
}

/*
    Clean up resources associated with the interrupt module by disabling all
    interrupts, removing all interrupt callbacks and freeing the Interrupt
    module state structure.
*/
aDIO_Error aDIO_Interrupt_Internal_CleanModule(aDIO_Handle Handle)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result;
    aDIO_Interrupt_Internal_Module_State* int_state;
    aDIO_Intrfc_Int_Control int_control;
    uint8 i;

    // Check if the Interrupt module has even been prepared
    if(!Handle->int_state)
        return ADIO_ERROR_CODE_SUCCESS;

    // Make a 'shortcut' to the Interrupt module state structure
    int_state = (aDIO_Interrupt_Internal_Module_State*)Handle->int_state;

    // Make sure the aDIO has interrupts
    if(Handle->dev_info.irq_number != 0)
    {
        // Remove all interrupt callbacks
        for(i = 0; i < ADIO_INTERRUPT_INT_TYPE_COUNT; ++i)
        {
            temp_result = aDIO_Interrupt_Internal_Callback_Remove(
                                                    Handle,
                                                    (aDIO_Interrupt_Int_Type)i);
            if(result == ADIO_ERROR_CODE_SUCCESS)
                result = temp_result;
        }

        // Disable all interrupts
        for(i = 0; i < ADIO_INTERRUPT_INT_TYPE_COUNT; ++i)
        {
            int_control.type = aDIO_Interrupt_Internal_ConvertType(
                                                    (aDIO_Interrupt_Int_Type)i);
            int_control.change_state = TRUE;
            int_control.enable = FALSE;

            temp_result = aDIO_General_Internal_CallIoctl(
                                                Handle,
                                                IOCTL_ADIO_INT_CONTROL,
                                                &int_control,
                                                sizeof(aDIO_Intrfc_Int_Control),
                                                NULL);
            if(result == ADIO_ERROR_CODE_SUCCESS)
                result = temp_result;
        }
    }

    // Free the memory for the Interrupt module state structure
    free(Handle->int_state);
    Handle->int_state = NULL;

    return result;
}

//============================================================================
// End of Interrupt Internal Functions
//============================================================================

//============================================================================
// End of Interrupt Internal Module
//============================================================================

//============================================================================
// End of Internal Module
//============================================================================

