/**
    @file

    @brief
        This file defines internal functions for the 'Interrupt' module of the
        aDIOLib.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Interrupt_Internal.h 55136 2011-08-18 18:13:56Z jstrouse $
*/

#ifndef __ADIOLIB_INTERRUPT_INTERNAL_H__
#define __ADIOLIB_INTERRUPT_INTERNAL_H__

#include "aDIOLib_Interrupt.h"

#ifdef __cplusplus
extern "C" {
#endif


//============================================================================
//! @defgroup aDIO_Internal aDIOLib Internal Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt_Internal aDIOLib Interrupt Internal Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt_Internal_Define aDIOLib Interrupt Internal Defines
//! @{
//============================================================================

/**
    Creates a type for the state of interrupt callback threads.
*/
typedef LONG aDIO_Interrupt_Internal_Thread_State;

/**
    Defines an aDIO_Interrupt_Internal_Thread_State value representing a thread
    whose callback is being installed.

    This is the value of a thread's state when it is created during the callback
    installation. If the thread starts without error, it sets its state to
    'running'. If an error occurs inside the thread while it is starting, it
    sets the error code, sets its state to 'error', cleans up and exits.

    After starting the thread, the installation function waits for the thread to
    go into either the 'running' state or the 'error' state.

    @note
        When a thread starts it can safely use internal library functions and
        data while the installation function waits for it to go to the 'running'
        or 'error' state,
*/
#define ADIO_INTERRUPT_INTERNAL_THREAD_STATE_INSTALLING 0

/**
    Defines an aDIO_Interrupt_Internal_Thread_State value representing a thread
    that is running successfully.

    If an error occurs inside a thread while it's in the 'running' state, it
    sets the error code, call's the user's callback, sets its state to 'error',
    and waits for its state to be set to 'removing'.
*/
#define ADIO_INTERRUPT_INTERNAL_THREAD_STATE_RUNNING    1

/**
    Defines an aDIO_Interrupt_Internal_Thread_State value representing a thread
    that has encountered an error.

    The thread sets the error code value in the thread data to describe the
    error that occurred.
*/
#define ADIO_INTERRUPT_INTERNAL_THREAD_STATE_ERROR      2

/**
    Defines an aDIO_Interrupt_Internal_Thread_State value representing a thread
    whose callback is being removed.

    When the callback removal function is called it sets the thread's
    termination event and then sets the thread's state to 'removing'. When the
    thread sees that its state has been set to 'removing' it begins to shut
    down. If the thread shuts down without error, the thread sets its state to
    'complete' before exiting. If an error occurs during shutdown, it sets the
    error code and sets the 'error' state before exiting.

    After setting the termination event, the removal function waits for the
    thread to go into either the 'complete' state or the 'error' state.

    @note
        When a thread's state has been set to 'removing' it can safely use
        internal library functions and data while the removal function waits
        for it to go to the 'complete' or 'error' state,
*/
#define ADIO_INTERRUPT_INTERNAL_THREAD_STATE_REMOVING   3

/**
    Defines an aDIO_Interrupt_Internal_Thread_State value representing a thread
    that has shut down without error.
*/
#define ADIO_INTERRUPT_INTERNAL_THREAD_STATE_COMPLETE   4

/**
    Defines the number of attempts that should be made to let a thread reach a
    certain state before timing out.
*/
#define ADIO_INTERRUPT_INTERNAL_THREAD_STATE_WAIT_LIMIT 100

/**
    Defines a time (in milliseconds) to wait between checks of a thread's state.
    This value must be greater than zero.
*/
#define ADIO_INTERRUPT_INTERNAL_THREAD_STATE_WAIT_TIME  5

/**
    Defines the number of attempts that should be made to let a thread close
    (after force terminating it) before timing out.
*/
#define ADIO_INTERRUPT_INTERNAL_FORCE_TERM_WAIT_LIMIT   100

/**
    Defines a time (in milliseconds) to wait between checks of a thread's
    closing after a forced termination. This value must be greater than zero.
*/
#define ADIO_INTERRUPT_INTERNAL_FORCE_TERM_WAIT_TIME    5

/**
    Defines a timeout (in milliseconds) for waiting on thread events to become
    set.
*/
#define ADIO_INTERRUPT_INTERNAL_EVENT_WAIT_TIME 1000

/**
    Defines the number of notification events that should be queued up in each
    interrupt callback thread. This value must be no less than two.
*/
#define ADIO_INTERRUPT_INTERNAL_QUEUE_DEPTH 4

//============================================================================
//! @} End of Interrupt Internal Defines
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt_Internal_Struct aDIOLib Interrupt Internal Structs
//! @{
//============================================================================

/**
    Holds information passed to an interrupt callback thread.
*/
typedef struct
{
    /**
        This is a handle to the aDIO device being controlled.

        @note
            This handle can only be used to log errors unless a thread has the
            library's global lock or is in the 'installing' or 'removing' state.
    */
    aDIO_Handle handle;

    /**
        This is the library-type of the interrupt that the callback is for.
    */
    aDIO_Interrupt_Int_Type type;

    /**
        This is the driver-type of the interrupt that the callback is for.
    */
    aDIO_Interrupt_Type driver_type;

    /**
        This is a handle to an event that, when set, causes the thread to exit.
    */
    HANDLE terminate_event;

    /**
        This is an array of parameter structures for the interrupt status IOCTL,
        used to queue interrupt notification requests.
    */
    aDIO_Intrfc_Int_Status int_status[ADIO_INTERRUPT_INTERNAL_QUEUE_DEPTH];

    /**
        This is an array of OVERLAPPED structures that is used when asking for
        notification of interrupts.

        @note
            These structures contain the events that are signaled when
            interrupts occur.
    */
    OVERLAPPED overlappeds[ADIO_INTERRUPT_INTERNAL_QUEUE_DEPTH];

    /**
        This value is set by the thread to denote its status.

        @note
            It should only be accessed by the Interlocked* functions.
    */
    aDIO_Interrupt_Internal_Thread_State state;

    /**
        If an error occurs while the thread is starting or stopping, this is the
        error that occurred.
    */
    aDIO_Error error_code;

    /**
        This is a function pointer to the user's callback function.
    */
    aDIO_Interrupt_Callback callback;

    /**
        This is the user-provided pointer passed into
        aDIO_Interrupt_InstallCallback.
    */
    void* user_pointer;

} aDIO_Interrupt_Internal_Thread_Data;

//============================================================================
//! @defgroup aDIO_Interrupt_Internal_State aDIOLib Interrupt Internal State
//! @{
//============================================================================

/**
    Holds information on the current state of a single interrupt type.
*/
typedef struct
{
    /**
        Marks if this interrupt type has a callback installed for it.

        @note All callbacks will removed if the aDIO is reset.
    */
    BOOL is_callback_installed;

    /**
        Holds a handle to this interrupt type's callback thread.
    */
    HANDLE thread_handle;

    /**
        Holds a pointer to the aDIO_Interrupt_Internal_Thread_Data structure
        which holds data associated with this interrupt type's callback thread.

        @note
            When a callback is installed, the memory needed for its thread data
            is allocated. When a callback is removed, the memory used by its
            thread data is freed.
    */
    aDIO_Interrupt_Internal_Thread_Data* thread_data;

} aDIO_Interrupt_Internal_Type_State;

/**
    Holds information on the current state of the Interrupt module.
*/
typedef struct
{
    /**
        Holds interrupt-specific state information.

        @note The driver interrupt type enumeration is used for indexing.
    */
    aDIO_Interrupt_Internal_Type_State types[ADIO_INT_TYPE_COUNT];

} aDIO_Interrupt_Internal_Module_State;

//============================================================================
//! @} End of Interrupt Internal State
//============================================================================

//============================================================================
//! @} End of Interrupt Internal Structures
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt_Internal_Func aDIOLib Interrupt Internal Functions
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt_Callback_Func aDIOLib Interrupt Callback Functions
//! @{
//============================================================================

/**
    @internal

    @brief
        Performs the work of installing an interrupt callback function.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Type

        The interrupt type to install a callback for.

    @param
        Callback

        The callback function to be called whenever the selected interrupt
        occurs.

    @param
        UserPtr

        An arbitrary pointer that will be passed to the callback function. This
        parameter can be NULL.

    @note
        If an error occurs while the callback thread is starting, this function
        will return the appropriate error code. The callback function will NOT
        be called.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_Callback_Install(
                                            aDIO_Handle             Handle,
                                            aDIO_Interrupt_Int_Type Type,
                                            aDIO_Interrupt_Callback Callback,
                                            void*                   UserPtr);

/**
    @internal

    @brief
        Allocates and prepares all resources needed by an interrupt callback
        thread.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Type

        The interrupt type the callback is being installed for.

    @param
        Callback

        The callback function to be called whenever the selected interrupt
        occurs.

    @param
        UserPtr

        An arbitrary pointer that will be passed to the callback function. This
        parameter can be NULL.

    @param
        ThreadData

        A pointer to a location that is filled with a pointer to a fully
        prepared thread data structure.

    @note
        If an error occurs during allocation, clean up will be done before
        returning.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_Callback_AllocateResources(
                            aDIO_Handle                             Handle,
                            aDIO_Interrupt_Int_Type                 Type,
                            aDIO_Interrupt_Callback                 Callback,
                            void*                                   UserPtr,
                            aDIO_Interrupt_Internal_Thread_Data**   ThreadData);

/**
    @internal

    @brief
        Waits, with a timeout, for the given thread to reach a specific thread
        state or to report an error.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        ThreadData

        A pointer to a thread data structure for the thread whose state is being
        checked.

    @param
        GoalState

        This is the thread state that is being waited for.

    @param
        ForceTerm

        A pointer to a location that is filled with FALSE if the given thread
        is in an expected state and can be terminated normally. If the given
        thread is in an unknown or unexpected state, this value is set to TRUE.

    @return
        If the state given by 'GoalState' is reached, ADIO_ERROR_CODE_SUCCESS is
        returned. If the thread enters the error state, the resulting error code
        is returned. Otherwise, ADIO_ERROR_CODE_UNEXPECTED is returned.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_Callback_WaitThreadState(
                            aDIO_Handle                             Handle,
                            aDIO_Interrupt_Internal_Thread_Data*    ThreadData,
                            aDIO_Interrupt_Internal_Thread_State    GoalState,
                            BOOL*                                   ForceTerm);

/**
    @internal

    @brief
        Spins until the 'global lock' is obtained, the thread's termination
        event is set, or until an error occurs.

    @param
        ThreadData

        A pointer to the aDIO_Interrupt_Internal_Thread_Data structure which was
        passed into the callback thread that is obtaining the global lock.

    @param
        StopThread

        A pointer to a location that is set to FALSE if the global lock is
        obtained. If the thread's termination event is set, this value will be
        set to TRUE and ADIO_ERROR_CODE_SUCCESS will be returned. If an error
        occurs, this value will be set to TRUE and the appropriate error code
        will be returned.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_Callback_ObtainGlobalLock(
                            aDIO_Interrupt_Internal_Thread_Data*    ThreadData,
                            BOOL*                                   StopThread);

/**
    @internal

    @brief
        Handles getting notification results and then re-queuing the
        notification.

    @param
        ThreadData

        A pointer to the aDIO_Interrupt_Internal_Thread_Data structure for the
        thread which received notification.

    @param
        CallbackInfo

        A pointer to a location that is filled with the notification-specific
        information from the notification being handled.

    @param
        CurrentEvent

        The index into the thread data's interrupt status array and OVERLAPPED
        array for the notification event being handled.

    @note
        Even though this function is called from the callback thread context,
        the global lock for the library must be obtained before calling this
        function.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_Callback_HandleNotify(
                        aDIO_Interrupt_Internal_Thread_Data*    ThreadData,
                        aDIO_Interrupt_Callback_Info*           CallbackInfo,
                        uint32                                  CurrentEvent);

/**
    @internal

    @brief
        Performs the core operations of the callback thread while in its
        'running' state.

    @param
        ThreadData

        A pointer to an aDIO_Interrupt_Internal_Thread_Data structure which
        contains the information needed by the thread.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_Callback_ThreadCore(
                            aDIO_Interrupt_Internal_Thread_Data* ThreadData);

/**
    @internal

    @brief
        Created in a new thread to monitor a specific interrupt and notify the
        user when it occurs.

    @param
        ThreadData

        A pointer to an aDIO_Interrupt_Internal_Thread_Data structure which
        contains the information needed by the thread.

    @warning
        This is an internal function and should not be called directly.

*/
DWORD WINAPI aDIO_Interrupt_Internal_Callback_ThreadFunc(void* ThreadData);

/**
    @internal

    @brief
        Clears all interrupt notification events from a callback thread's
        notification queue.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        ThreadData

        A pointer to an aDIO_Interrupt_Internal_Thread_Data structure for the
        callback thread whose notification queue will be cleared.

    @note
        This function can be used on thread data that does not have all the
        notification events created or queued.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_Callback_ClearNotificationQueue(
                            aDIO_Handle                             Handle,
                            aDIO_Interrupt_Internal_Thread_Data*    ThreadData);

/**
    @internal

    @brief
        Cleans up thread-related resources.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        ThreadHandle

        An interrupt callback thread handle to close. This parameter may be
        NULL.

    @param
        ThreadData

        A pointer to a thread data structure to clean up. This parameter may be
        NULL.

    @param
        ForceTerm

        If TRUE, the thread whose handle is given by 'ThreadHandle' will be
        forcibly terminated before the 'ThreadHandle' itself is closed.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_Callback_CleanupResources(
                        aDIO_Handle                             Handle,
                        HANDLE                                  ThreadHandle,
                        aDIO_Interrupt_Internal_Thread_Data*    ThreadData,
                        BOOL                                    ForceTerm);

/**
    @internal

    @brief
        Performs the work of removing an interrupt callback function.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Type

        The interrupt type to remove a callback for.

    @note
        If an error occurs while the callback thread is shutting down, this
        function will return the appropriate error code. The callback function
        will NOT be called.

    @note
        This function removes the callback as completely as possible, even if
        an error occurs. So, if this function does return an error, there is no
        need to call it again for the same type.

    @note
        If there is no callback to be removed for the given interrupt type, the
        function will simply return ADIO_ERROR_CODE_SUCCESS.

    @warning
        Make sure the user's callback is not calling library functions when it
        is being removed or a deadlock may occur.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_Callback_Remove(
                                            aDIO_Handle             Handle,
                                            aDIO_Interrupt_Int_Type Type);

//============================================================================
//! @} End of Interrupt Callback Functions
//============================================================================

/**
    @internal

    @brief
        Takes a library interrupt type and converts it into a driver interrupt
        type.

    @param
        Type

        The library interrupt type.

    @return
        The driver-compatible interrupt type corresponding to the type value
        passed in.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Interrupt_Type aDIO_Interrupt_Internal_ConvertType(
                                                aDIO_Interrupt_Int_Type Type);

/**
    @internal

    @brief
        Prepares the Interrupt module for operation.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_PrepareModule(aDIO_Handle Handle);

/**
    @internal

    @brief
        Resets the Interrupt module back to its default state.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_ResetModule(aDIO_Handle Handle);

/**
    @internal

    @brief
        Clean up resources associated with the interrupt module by disabling all
        interrupts, removing all interrupt callbacks and freeing the Interrupt
        module state structure.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Interrupt_Internal_CleanModule(aDIO_Handle Handle);

//============================================================================
//! @} End of Interrupt Internal Functions
//============================================================================

//============================================================================
//! @defgroup aDIO_Interrupt_Internal_Macro aDIOLib Interrupt Internal Macros
//! @{
//============================================================================

/**
    @brief
        Checks the validity of an aDIO_Interrupt_Int_Type value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Type

        The value to check.

    @warning
        If the value is not valid, an appropriate error message will be logged
        and the function using this macro will immediately return
        ADIO_ERROR_CODE_BAD_PARAM.

*/
#define ADIO_INTERRUPT_INTERNAL_CHECK_INT_TYPE(Handle, Type) \
    switch(Type)\
    {\
    case ADIO_INTERRUPT_INT_TYPE_STROBE:\
    case ADIO_INTERRUPT_INT_TYPE_EVENT:\
    case ADIO_INTERRUPT_INT_TYPE_MATCH:\
        break;\
    default:\
        return ADIO_LOG_INTERNAL_LOG_BAD_PARAM_INT(\
                                                Handle,\
                                                ADIO_ERROR_CATEGORY_INTERRUPT,\
                                                Type);\
    }

//============================================================================
//! @} End of Interrupt Internal Macros
//============================================================================

//============================================================================
//! @} End of Interrupt Internal Module
//============================================================================

//============================================================================
//! @} End of Internal Module
//============================================================================


#ifdef __cplusplus
}
#endif


#endif // __ADIOLIB_INTERRUPT_INTERNAL_H__
