/**
    @file

    @brief
        This file implements functions for the internal 'Log' module of the
        aDIOLib.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Log_Internal.c 55027 2011-08-15 20:24:09Z jstrouse $
*/

#include "aDIOLib_Log_Internal.h"


//============================================================================
// aDIOLib Internal Module
//============================================================================

//============================================================================
// aDIOLib Log Internal Module
//============================================================================

//============================================================================
// aDIOLib Log Internal Functions
//============================================================================

/*
    Edits the Registry to mark the aDIOLib DLL as an Event Log source.
*/
BOOL aDIO_Log_Internal_RegisterEventSource(HINSTANCE Instance)
{
    char key_path[100] =
        "SYSTEM\\CurrentControlSet\\Services\\Eventlog\\Application\\aDIOLib";
    char dll_path[MAX_PATH];
    DWORD dll_path_len, supported, cat_count;
    HKEY sub_key;

    // Get the full path of the DLL
    dll_path_len = GetModuleFileName(Instance, dll_path, MAX_PATH);
    if(!dll_path_len)
        return FALSE;

    // Create the event log key
    if(RegCreateKeyEx(  HKEY_LOCAL_MACHINE,
                        key_path,
                        0,
                        NULL,
                        REG_OPTION_NON_VOLATILE,
                        KEY_WRITE,
                        NULL,
                        &sub_key,
                        NULL))
    {
        return FALSE;
    }

    // Set the name of the message file
    if(RegSetValueEx(   sub_key,
                        "EventMessageFile",
                        0,
                        REG_EXPAND_SZ,
                        (LPBYTE)dll_path,
                        dll_path_len + 1))
    {
        RegCloseKey(sub_key);
        return FALSE;
    }

    // Set the supported event types
    supported = EVENTLOG_ERROR_TYPE |
                EVENTLOG_WARNING_TYPE |
                EVENTLOG_INFORMATION_TYPE |
                EVENTLOG_SUCCESS;
    if(RegSetValueEx(   sub_key,
                        "TypesSupported",
                        0,
                        REG_DWORD,
                        (LPBYTE)&supported,
                        sizeof(DWORD)))
    {
        RegCloseKey(sub_key);
        return FALSE;
    }

    // Set the name of the category file
    if(RegSetValueEx(   sub_key,
                        "CategoryMessageFile",
                        0,
                        REG_EXPAND_SZ,
                        (LPBYTE)dll_path,
                        dll_path_len + 1))
    {
        RegCloseKey(sub_key);
        return FALSE;
    }

    // Set the number of categories
    cat_count = ADIO_ERROR_CATEGORY_COUNT;
    if(RegSetValueEx(   sub_key,
                        "CategoryCount",
                        0,
                        REG_DWORD,
                        (LPBYTE)&cat_count,
                        sizeof(DWORD)))
    {
        RegCloseKey(sub_key);
        return FALSE;
    }

    // Clean up the registry key
    RegCloseKey(sub_key);

    return TRUE;
}

/*
    Fills a buffer with the name of an IOCTL code.
*/
void aDIO_Log_Internal_GetIoctlName(DWORD   IoctlCode,
                                    char*   Buffer,
                                    uint8   BufferSize)
{
    // 'Clear' the buffer
    Buffer[0] = '\0';

    // Copy in the appropriate IOCTL name
    switch(IoctlCode)
    {
    case IOCTL_ADIO_QUERY_BOARD:
        strcpy_s(Buffer, BufferSize, "IOCTL_ADIO_QUERY_BOARD");
        break;
    case IOCTL_ADIO_BOARD_RESET:
        strcpy_s(Buffer, BufferSize, "IOCTL_ADIO_BOARD_RESET");
        break;
    case IOCTL_ADIO_REGISTER_OP:
        strcpy_s(Buffer, BufferSize, "IOCTL_ADIO_REGISTER_OP");
        break;
    case IOCTL_ADIO_GUARD_BIT_CONTROL:
        strcpy_s(Buffer, BufferSize, "IOCTL_ADIO_GUARD_BIT_CONTROL");
        break;
    case IOCTL_ADIO_INT_CONTROL:
        strcpy_s(Buffer, BufferSize, "IOCTL_ADIO_INT_CONTROL");
        break;
    case IOCTL_ADIO_INT_STATUS:
        strcpy_s(Buffer, BufferSize, "IOCTL_ADIO_INT_STATUS");
        break;
    case IOCTL_ADIO_WAKE_CONTROL:
        strcpy_s(Buffer, BufferSize, "IOCTL_ADIO_WAKE_CONTROL");
        break;
    default:
        sprintf_s(Buffer, BufferSize, "0x%X", IoctlCode);
    }
}

/*
    Composes and logs an error message.
*/
aDIO_Error aDIO_Log_Internal_LogError(  aDIO_Handle     Handle,
                                        aDIO_Error      Error,
                                        aDIO_Error_Cat  Category,
                                        uint8           Severity,
                                        const char*     Filename,
                                        uint32          LineNumber,
                                        char*           FormatString,
                                        ...)
{
    HANDLE log_handle;
    WORD event_log_type;
    char final_buff[ADIO_ERROR_MAX_MSG_LENGTH];
    char temp_buff[ADIO_ERROR_MAX_MSG_LENGTH];
    char default_msg[] = "Error message creation failed.";
    va_list arguments;
    int result;
    char* msg;

    // Enter the logging function critical section
    if(Handle != NULL)
        EnterCriticalSection(&Handle->log_section);

    msg = default_msg;
    do
    {
        // Create the main part of the message string
        va_start(arguments, FormatString);
        result = vsprintf_s(final_buff,
                            ADIO_ERROR_MAX_MSG_LENGTH,
                            FormatString,
                            arguments);
        va_end(arguments);
        if(result <= 0)
            break;

        // Add the file and line number to the message string
        result = sprintf_s( temp_buff,
                            ADIO_ERROR_MAX_MSG_LENGTH,
                            "\n(In %s, line %d)",
                            Filename,
                            LineNumber);
        if(result <= 0)
            break;

        // Append the file/line string to the main string
        if(strcat_s(final_buff, ADIO_ERROR_MAX_MSG_LENGTH, temp_buff) != 0)
            break;

        // Mark the composed message as the one to use
        msg = final_buff;

    } while(0);

    // If the device descriptor doesn't have a 'last error message', copy this
    //  message into it
    if(Handle != NULL && strlen(Handle->last_error_msg) <= 0)
    {
        if(strcpy_s(Handle->last_error_msg, ADIO_ERROR_MAX_MSG_LENGTH, msg))
            printf("Error message could not be copied to device descriptor.\n");
    }

    // Get the 'event log type' for ReportEvent from Severity
    switch(Severity)
    {
    case ADIO_ERROR_SEVERITY_SUCCESS:
        event_log_type = EVENTLOG_SUCCESS;
        break;
    case ADIO_ERROR_SEVERITY_INFORMATIONAL:
        event_log_type = EVENTLOG_INFORMATION_TYPE;
        break;
    case ADIO_ERROR_SEVERITY_WARNING:
        event_log_type = EVENTLOG_WARNING_TYPE;
        break;
    default:
        event_log_type = EVENTLOG_ERROR_TYPE;
    }

    // Get a handle to the Event Log
    log_handle = RegisterEventSource(NULL, "aDIOLib");
    if(log_handle == NULL)
    {
        printf("Could not register the event source. GLE %d\n", GetLastError());
    }
    else
    {
        // Report the event
        if(!ReportEvent(log_handle,
                        event_log_type,
                        Category,
                        Error,
                        NULL,
                        1,
                        0,
                        &msg,
                        NULL))
        {
            printf("Could not report the event. GLE %d\n", GetLastError());
        }

        // Clean up the handle to the Event Log
        DeregisterEventSource(log_handle);
    }

    // Leave the logging function critical section
    if(Handle != NULL)
        LeaveCriticalSection(&Handle->log_section);

    return Error;
}

//============================================================================
// End of Log Internal Functions
//============================================================================

//============================================================================
// End of Log Internal Module
//============================================================================

//============================================================================
// End of Internal Module
//============================================================================

