/**
    @file

    @brief
        This file defines functions for the internal 'Log' module of the
        aDIOLib.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Log_Internal.h 54547 2011-07-15 20:46:51Z jstrouse $
*/

#ifndef __ADIOLIB_LOG_INTERNAL_H__
#define __ADIOLIB_LOG_INTERNAL_H__

#include "aDIOLib_General.h"

#ifdef __cplusplus
extern "C" {
#endif


//============================================================================
//! @defgroup aDIO_Internal aDIOLib Internal Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Log_Internal aDIOLib Log Internal Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Log_Internal_Define aDIOLib Log Internal Defines
//! @{
//============================================================================

/**
    Defines the maximum length for the name of an IOCTL.
*/
#define ADIO_LOG_INTERNAL_MAX_IOCTL_NAME_LENGTH 30

//============================================================================
//! @} End of Log Internal Defines
//============================================================================

//============================================================================
//! @defgroup aDIO_Log_Internal_Func aDIOLib Log Internal Functions
//! @{
//============================================================================

/**
    @internal

    @brief
        Edits the Registry to mark the aDIOLib DLL as an Event Log source.

    @param
        Instance

        The module handle for the DLL that is passed into DllMain.

    @return
        Returns TRUE on success and FALSE on failure.

    @warning
        This is an internal function and should not be called directly.

*/
BOOL aDIO_Log_Internal_RegisterEventSource(HINSTANCE Instance);

/**
    @internal

    @brief
        Fills a buffer with the name of an IOCTL code.

    @param
        IoctlCode

        The IOCTL to get the name of.

    @param
        Buffer

        The buffer to copy the name into. This buffer must be at least
        ADIO_LOG_INTERNAL_MAX_IOCTL_NAME_LENGTH characters in size.

    @param
        BufferSize

        The size, in bytes, of the buffer pointed to by 'Buffer'.

    @warning
        This is an internal function and should not be called directly.

*/
void aDIO_Log_Internal_GetIoctlName(DWORD   IoctlCode,
                                    char*   Buffer,
                                    uint8   BufferSize);

/**
    @internal

    @brief
        Composes and logs an error message.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Error

        The error code for the error that occurred.

    @param
        Category

        The category/module where the error occurred.

    @param
        Severity

        The severity/classification of the error that occurred.

    @param
        Filename

        The name of the file where the error occurred.

    @param
        LineNumber

        The line number where the error occurred.

    @param
        FormatString

        A pointer to a printf-style format string used to create the error
        message.

    @return
        Returns the error code passed into 'Error'.

    @note
        This function takes a variable number of parameters. Any parameters
        passed in, beyond those documented, are used to fill in the printf-style
        format string pointed to by 'FormatString'.

    @warning
        This is an internal function and should not be called directly.

*/
aDIO_Error aDIO_Log_Internal_LogError(  aDIO_Handle     Handle,
                                        aDIO_Error      Error,
                                        aDIO_Error_Cat  Category,
                                        uint8           Severity,
                                        const char*     Filename,
                                        uint32          LineNumber,
                                        char*           FormatString,
                                        ...);

//============================================================================
//! @} End of Log Internal Functions
//============================================================================

//============================================================================
//! @defgroup aDIO_Log_Internal_Macro aDIOLib Log Internal Macros
//! @{
//============================================================================

/**
    @brief
        Defines a macro used to log a message for any type of error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Error

        This is the error code of the message to be logged.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Severity

        The severity of the message being logged. See aDIO_Error.h for details.

    @param
        Message

        A string literal or a pointer to a string, giving a printf-style format
        string used to create the message.

    @return
        Returns the error code passed into 'Error'.

    @note
        This macro takes a variable number of parameters. Any parameters
        passed in, beyond those documented, are used to fill in the format
        string, 'Message'.

*/
#define ADIO_LOG_INTERNAL_LOG_ANY(  Handle,\
                                    Error,\
                                    Category,\
                                    Severity,\
                                    Message,\
                                    ...)\
            aDIO_Log_Internal_LogError( Handle,\
                                        Error,\
                                        Category,\
                                        Severity,\
                                        __FILE__,\
                                        __LINE__,\
                                        Message,\
                                        __VA_ARGS__)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_SUCCESS error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @return
        Returns ADIO_ERROR_CODE_SUCCESS.

*/
#define ADIO_LOG_INTERNAL_LOG_SUCCESS(Handle, Category) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_SUCCESS,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_SUCCESS,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_SUCCESS)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_INFO error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Message

        A string literal or a pointer to a string, giving the info message to
        log.

    @return
        Returns ADIO_ERROR_CODE_INFO.

*/
#define ADIO_LOG_INTERNAL_LOG_INFO(Handle, Category, Message) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_INFO,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_INFORMATIONAL,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_INFO,\
                                        Message)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_WARNING error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Message

        A string literal or a pointer to a string, giving the warning message
        to log.

    @return
        Returns ADIO_ERROR_CODE_WARNING.

*/
#define ADIO_LOG_INTERNAL_LOG_WARNING(Handle, Category, Message) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_WARNING,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_WARNING,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_WARNING,\
                                        Message)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_UNEXPECTED error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Message

        A string literal or a pointer to a string, giving the error message to
        log.

    @return
        Returns ADIO_ERROR_CODE_UNEXPECTED.

*/
#define ADIO_LOG_INTERNAL_LOG_UNEXPECTED(Handle, Category, Message) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_UNEXPECTED,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_UNEXPECTED,\
                                        Message)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_BAD_PARAM error code
        when a bad (NULL) pointer parameter is encountered.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Param

        The parameter with the bad value.

    @return
        Returns ADIO_ERROR_CODE_BAD_PARAM.

*/
#define ADIO_LOG_INTERNAL_LOG_BAD_PARAM_PTR(Handle, Category, Param) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_BAD_PARAM,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_BAD_PARAM_PTR,\
                                        __FUNCTION__,\
                                        #Param)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_BAD_PARAM error code
        when a bad integer parameter is encountered.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Param

        The parameter with the bad value.

    @return
        Returns ADIO_ERROR_CODE_BAD_PARAM.

*/
#define ADIO_LOG_INTERNAL_LOG_BAD_PARAM_INT(Handle, Category, Param) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_BAD_PARAM,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_BAD_PARAM_INT,\
                                        __FUNCTION__,\
                                        #Param,\
                                        Param)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_BAD_PARAM error code
        when a bad float parameter is encountered.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Param

        The parameter with the bad value.

    @return
        Returns ADIO_ERROR_CODE_BAD_PARAM.

*/
#define ADIO_LOG_INTERNAL_LOG_BAD_PARAM_FLOAT(Handle, Category, Param) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_BAD_PARAM,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_BAD_PARAM_FLOAT,\
                                        __FUNCTION__,\
                                        #Param,\
                                        Param)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_BAD_PARAM error code
        when a bad string parameter is encountered.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Param

        The parameter with the bad value.

    @return
        Returns ADIO_ERROR_CODE_BAD_PARAM.

*/
#define ADIO_LOG_INTERNAL_LOG_BAD_PARAM_STR(Handle, Category, Param) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_BAD_PARAM,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_BAD_PARAM_STR,\
                                        __FUNCTION__,\
                                        #Param,\
                                        Param)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_NOT_CONFIG error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Module

        A string literal or a pointer to a string, giving the name of the
        module or functionality that is not configured.

    @return
        Returns ADIO_ERROR_CODE_NOT_CONFIG.

*/
#define ADIO_LOG_INTERNAL_LOG_NOT_CONFIG(Handle, Category, Module) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_NOT_CONFIG,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_NOT_CONFIG,\
                                        Module)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_NOT_ENABLE error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Module

        A string literal or a pointer to a string, giving the name of the
        module or functionality that is not enabled.

    @return
        Returns ADIO_ERROR_CODE_NOT_ENABLE.

*/
#define ADIO_LOG_INTERNAL_LOG_NOT_ENABLE(Handle, Category, Module) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_NOT_ENABLE,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_NOT_ENABLE,\
                                        Module)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_IN_USE error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Module

        A string literal or a pointer to a string, giving the name of the
        module or functionality that is already in use.

    @return
        Returns ADIO_ERROR_CODE_IN_USE.

*/
#define ADIO_LOG_INTERNAL_LOG_IN_USE(Handle, Category, Module) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_IN_USE,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_IN_USE,\
                                        Module)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_NO_MEMORY error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Size

        The size, in bytes, of the allocation that failed.

    @return
        Returns ADIO_ERROR_CODE_NO_MEMORY.

*/
#define ADIO_LOG_INTERNAL_LOG_NO_MEMORY(Handle, Category, Size) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_NO_MEMORY,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_NO_MEMORY,\
                                        Size)

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_WIN_API error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Function

        A string literal or a pointer to a string, giving the name of the
        WinAPI function that failed.

    @return
        Returns ADIO_ERROR_CODE_WIN_API.

    @note
        This macro automatically calls GetLastError for logging purposes.

*/
#define ADIO_LOG_INTERNAL_LOG_WIN_API(Handle, Category, Function) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_WIN_API,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_WIN_API,\
                                        Function,\
                                        GetLastError())

/**
    @brief
        Defines a macro used to log the ADIO_ERROR_CODE_IOCTL_FAIL error code.

    @param
        Handle

        A valid handle to the aDIO device to be controlled. This parameter can
        be NULL. If it is the message will still be logged but it will not be
        recorded in a device descriptor to be retrieved by the user.

    @param
        Category

        The category/module where the message logging is occurring.

    @param
        Ioctl

        A string literal or a pointer to a string, giving the name of the
        IOCTL that failed.

    @return
        Returns ADIO_ERROR_CODE_IOCTL_FAIL.

    @note
        This macro automatically calls GetLastError for logging purposes.

*/
#define ADIO_LOG_INTERNAL_LOG_IOCTL_FAIL(Handle, Category, Ioctl) \
            aDIO_Log_Internal_LogError( Handle,\
                                        ADIO_ERROR_CODE_IOCTL_FAIL,\
                                        Category,\
                                        ADIO_ERROR_SEVERITY_ERROR,\
                                        __FILE__,\
                                        __LINE__,\
                                        ADIO_ERROR_MSG_IOCTL_FAIL,\
                                        Ioctl,\
                                        GetLastError())

//============================================================================
//! @} End of Log Internal Macros
//============================================================================

//============================================================================
//! @} End of Log Internal Module
//============================================================================

//============================================================================
//! @} End of Internal Module
//============================================================================


#ifdef __cplusplus
}
#endif

#endif // __ADIOLIB_LOG_INTERNAL_H__
