/**
    @file

    @brief
        This file implements the Port Control API for the aDIO dll.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Port.c 55052 2011-08-16 19:33:01Z jstrouse $
*/

#include "aDIOLib_Port.h"
#include "aDIOLib_Port_Internal.h"
#include "aDIOLib_General_Internal.h"
#include "aDIOLib_Log_Internal.h"
#include "aDIOLib_RegMap_Internal.h"


//============================================================================
// aDIOLib API
//============================================================================

//============================================================================
// aDIOLib Port Module
//============================================================================

//============================================================================
// aDIOLib Port Functions
//============================================================================

/*
    Reads the value from a digital IO port.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_ReadPort(   aDIO_Handle         Handle,
                                        aDIO_Port_DIO_Port  Port,
                                        uint8*              Value)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Intrfc_Reg_Op_Size size;
    uint8 block;
    uint32 offset, data;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_PORT);
    ADIO_PORT_INTERNAL_CHECK_PORT(Handle, Port);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle, Value, ADIO_ERROR_CATEGORY_PORT);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    // Get the port parameters
    if(Port == ADIO_PORT_DIO_PORT_0)
    {
        size = ADIO_REGMAP_INTERNAL_PORT0_SIZE;
        block = ADIO_REGMAP_INTERNAL_PORT0_BLK;
        offset = ADIO_REGMAP_INTERNAL_PORT0_OFFSET;
    }
    else
    {
        size = ADIO_REGMAP_INTERNAL_PORT1_SIZE;
        block = ADIO_REGMAP_INTERNAL_PORT1_BLK;
        offset = ADIO_REGMAP_INTERNAL_PORT1_OFFSET;
    }

    // Read the port
    result = aDIO_General_Internal_RegOp(   Handle,
                                            ADIO_INTRFC_REG_OP_READ,
                                            size,
                                            block,
                                            offset,
                                            &data,
                                            0);
    if(result == ADIO_ERROR_CODE_SUCCESS)
        *Value = (uint8)data;

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Writes a value to a digital IO port.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_WritePort(  aDIO_Handle         Handle,
                                        aDIO_Port_DIO_Port  Port,
                                        uint8               Value)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Intrfc_Reg_Op_Size size;
    uint8 block;
    uint32 offset, data;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_PORT);
    ADIO_PORT_INTERNAL_CHECK_PORT(Handle, Port);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    // Get the port parameters
    if(Port == ADIO_PORT_DIO_PORT_0)
    {
        size = ADIO_REGMAP_INTERNAL_PORT0_SIZE;
        block = ADIO_REGMAP_INTERNAL_PORT0_BLK;
        offset = ADIO_REGMAP_INTERNAL_PORT0_OFFSET;
    }
    else
    {
        size = ADIO_REGMAP_INTERNAL_PORT1_SIZE;
        block = ADIO_REGMAP_INTERNAL_PORT1_BLK;
        offset = ADIO_REGMAP_INTERNAL_PORT1_OFFSET;
    }

    // Write to the port
    data = Value;
    result = aDIO_General_Internal_RegOp(   Handle,
                                            ADIO_INTRFC_REG_OP_WRITE,
                                            size,
                                            block,
                                            offset,
                                            &data,
                                            0);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Gets the current level of a digital IO strobe.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_ReadStrobe( aDIO_Handle             Handle,
                                        aDIO_Port_Strobe        Strobe,
                                        aDIO_Port_Strobe_Lev*   Level)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    uint32 data, mask;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_PORT);
    ADIO_PORT_INTERNAL_CHECK_STROBE(Handle, Strobe);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle, Level, ADIO_ERROR_CATEGORY_PORT);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    // Get the register mask for the strobe
    if(Strobe == ADIO_PORT_STROBE_0)
        mask = ADIO_REGMAP_INTERNAL_DIOCONTROL_STROBE0_MASK;
    else
        mask = ADIO_REGMAP_INTERNAL_DIOCONTROL_STROBE1_MASK;

    // Read the strobe result
    result = aDIO_General_Internal_RegOp(
                                        Handle,
                                        ADIO_INTRFC_REG_OP_READ,
                                        ADIO_REGMAP_INTERNAL_DIOCONTROL_SIZE,
                                        ADIO_REGMAP_INTERNAL_DIOCONTROL_BLK,
                                        ADIO_REGMAP_INTERNAL_DIOCONTROL_OFFSET,
                                        &data,
                                        0);
    if(result == ADIO_ERROR_CODE_SUCCESS)
    {
        // Use the strobe mask to determine the strobe state
        if(data & mask)
            *Level = ADIO_PORT_STROBE_LEV_HIGH;
        else
            *Level = ADIO_PORT_STROBE_LEV_LOW;
    }

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Gets the current 'direction' (input or output) of the digital IO ports.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_GetDirection(   aDIO_Handle         Handle,
                                            uint8*              Port0,
                                            aDIO_Port_Dir_P1*   Port1)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_PORT);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle, Port0, ADIO_ERROR_CATEGORY_PORT);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle, Port1, ADIO_ERROR_CATEGORY_PORT);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    do
    {
        // Get the direction of port 0
        result = aDIO_Port_Internal_GetP0Direction(Handle, Port0);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Get the direction of port 1
        result = aDIO_Port_Internal_GetP1Direction(Handle, Port1);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

    } while(0);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Sets the current 'direction' (input or output) of the digital IO ports.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_SetDirection(   aDIO_Handle         Handle,
                                            uint8               Port0,
                                            aDIO_Port_Dir_P1    Port1)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_PORT);
    ADIO_PORT_INTERNAL_CHECK_PORT1_DIR(Handle, Port1);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    do
    {
        // Set the direction of Port 0
        result = aDIO_Port_Internal_SetP0Direction(Handle, Port0);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Set the direction of Port 1
        result = aDIO_Port_Internal_SetP1Direction(Handle, Port1);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

    } while(0);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Retrieves the current Mask value.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_GetMask(aDIO_Handle Handle,
                                    uint8*      Mask)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_PORT);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle, Mask, ADIO_ERROR_CATEGORY_PORT);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    // Get the Mask
    result = aDIO_Port_Internal_GetMask(Handle, Mask);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Sets the desired Mask value.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_SetMask(aDIO_Handle Handle,
                                    uint8       Mask)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_PORT);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    // Set the Mask
    result = aDIO_Port_Internal_SetMask(Handle, Mask);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Retrieves the current Compare value.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_GetCompare( aDIO_Handle Handle,
                                        uint8*      Compare)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_PORT);
    ADIO_GENERAL_INTERNAL_CHECK_PTR(Handle, Compare, ADIO_ERROR_CATEGORY_PORT);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    // Get the Compare
    result = aDIO_Port_Internal_GetCompare(Handle, Compare);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

/*
    Sets the desired Compare value.
*/
ADIOLIB_API
aDIO_Error WINAPI aDIO_Port_SetCompare( aDIO_Handle Handle,
                                        uint8       Compare)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;

    // Check the parameters
    ADIO_GENERAL_INTERNAL_CHECK_HANDLE(Handle, ADIO_ERROR_CATEGORY_PORT);

    // Acquire the global lock
    EnterCriticalSection(&Handle->global_lock);

    // Set the Compare
    result = aDIO_Port_Internal_SetCompare(Handle, Compare);

    // Release the global lock
    LeaveCriticalSection(&Handle->global_lock);

    return result;
}

//============================================================================
// End of Port Functions
//============================================================================

//============================================================================
// End of Port Module
//============================================================================

//============================================================================
// End of API
//============================================================================

