/**
    @file

    @brief
        This file implements internal functions for the 'Port' module of the
        aDIOLib.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Port_Internal.c 55052 2011-08-16 19:33:01Z jstrouse $
*/

#include "aDIOLib_Port_Internal.h"
#include "aDIOLib_General_Internal.h"
#include "aDIOLib_Log_Internal.h"
#include "aDIOLib_RegMap_Internal.h"


//============================================================================
// aDIOLib Internal Module
//============================================================================

//============================================================================
// aDIOLib Port Internal Module
//============================================================================

//============================================================================
// aDIOLib Port Internal Functions
//============================================================================

/*
    Gets the current 'direction' (input or output) of Port 0.
*/
aDIO_Error aDIO_Port_Internal_GetP0Direction(   aDIO_Handle Handle,
                                                uint8*      Port0)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result;
    uint32 data;

    do
    {
        // Lock the Multi-function register to Port 0 Dir mode
        result = aDIO_General_Internal_LockMultiFunction(
                                    Handle,
                                    TRUE,
                                    ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_P0DIR);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Read the Multi-function register to get the direction of port 0
        result = aDIO_General_Internal_RegOp(
                                        Handle,
                                        ADIO_INTRFC_REG_OP_READ,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_SIZE,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_BLK,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_OFFSET,
                                        &data,
                                        0);

        // Copy out Port 0's direction
        if(result == ADIO_ERROR_CODE_SUCCESS)
            *Port0 = (uint8)data;

        // Unlock the Multi-function register
        temp_result = aDIO_General_Internal_LockMultiFunction(Handle, FALSE, 0);
        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = temp_result;

    } while(0);

    return result;
}

/*
    Gets the current 'direction' (input or output) of Port 1.
*/
aDIO_Error aDIO_Port_Internal_GetP1Direction(   aDIO_Handle         Handle,
                                                aDIO_Port_Dir_P1*   Port1)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    uint32 data;

    // Read the direction of port 1
    result = aDIO_General_Internal_RegOp(
                                        Handle,
                                        ADIO_INTRFC_REG_OP_READ,
                                        ADIO_REGMAP_INTERNAL_DIOCONTROL_SIZE,
                                        ADIO_REGMAP_INTERNAL_DIOCONTROL_BLK,
                                        ADIO_REGMAP_INTERNAL_DIOCONTROL_OFFSET,
                                        &data,
                                        0);
    if(result == ADIO_ERROR_CODE_SUCCESS)
    {
        // Use a mask to get the direction of port 1
        if(data & ADIO_REGMAP_INTERNAL_DIOCONTROL_P1DIR_MASK)
            *Port1 = ADIO_PORT_DIR_P1_OUTPUT;
        else
            *Port1 = ADIO_PORT_DIR_P1_INPUT;
    }

    return result;
}

/*
    Sets the current 'direction' (input or output) of Port 0.
*/
aDIO_Error aDIO_Port_Internal_SetP0Direction(   aDIO_Handle Handle,
                                                uint8       Port0)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result;
    uint32 data;

    do
    {
        // Lock the Multi-function register to Port 0 Dir mode
        result = aDIO_General_Internal_LockMultiFunction(
                                    Handle,
                                    TRUE,
                                    ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_P0DIR);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Write the Multi-function register to set the direction of port 0
        data = Port0;
        result = aDIO_General_Internal_RegOp(
                                        Handle,
                                        ADIO_INTRFC_REG_OP_WRITE,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_SIZE,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_BLK,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_OFFSET,
                                        &data,
                                        0);

        // Unlock the Multi-function register
        temp_result = aDIO_General_Internal_LockMultiFunction(Handle, FALSE, 0);
        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = temp_result;

    } while(0);

    return result;
}

/*
    Sets the current 'direction' (input or output) of Port 1.
*/
aDIO_Error aDIO_Port_Internal_SetP1Direction(   aDIO_Handle         Handle,
                                                aDIO_Port_Dir_P1    Port1)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    uint32 data;

    // Get the bit value for port 1's direction
    if(Port1 == ADIO_PORT_DIR_P1_INPUT)
        data = ADIO_REGMAP_INTERNAL_DIOCONTROL_P1DIR_INPUT;
    else
        data = ADIO_REGMAP_INTERNAL_DIOCONTROL_P1DIR_OUTPUT;

    // Write the direction of port 1
    result = aDIO_General_Internal_RegOp(
                                    Handle,
                                    ADIO_INTRFC_REG_OP_SAFEWRITE,
                                    ADIO_REGMAP_INTERNAL_DIOCONTROL_SIZE,
                                    ADIO_REGMAP_INTERNAL_DIOCONTROL_BLK,
                                    ADIO_REGMAP_INTERNAL_DIOCONTROL_OFFSET,
                                    &data,
                                    ADIO_REGMAP_INTERNAL_DIOCONTROL_P1DIR_MASK);

    return result;
}

/*
    Retrieves the current Mask value.
*/
aDIO_Error aDIO_Port_Internal_GetMask(  aDIO_Handle Handle,
                                        uint8*      Mask)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result;
    uint32 data;

    do
    {
        // Lock the Multi-function register to Mask mode
        result = aDIO_General_Internal_LockMultiFunction(
                                    Handle,
                                    TRUE,
                                    ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_MASK);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Read the Multi-function register to get mask value
        result = aDIO_General_Internal_RegOp(
                                        Handle,
                                        ADIO_INTRFC_REG_OP_READ,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_SIZE,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_BLK,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_OFFSET,
                                        &data,
                                        0);

        // Copy out the Mask value
        if(result == ADIO_ERROR_CODE_SUCCESS)
            *Mask = (uint8)data;

        // Unlock the Multi-function register
        temp_result = aDIO_General_Internal_LockMultiFunction(Handle, FALSE, 0);
        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = temp_result;

    } while(0);

    return result;
}

/*
    Sets the desired Mask value.
*/
aDIO_Error aDIO_Port_Internal_SetMask(  aDIO_Handle Handle,
                                        uint8       Mask)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result;
    uint32 data;

    do
    {
        // Lock the Multi-function register to Mask mode
        result = aDIO_General_Internal_LockMultiFunction(
                                    Handle,
                                    TRUE,
                                    ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_MASK);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;


        // Write the Multi-function register to set mask value
        data = Mask;
        result = aDIO_General_Internal_RegOp(
                                        Handle,
                                        ADIO_INTRFC_REG_OP_WRITE,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_SIZE,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_BLK,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_OFFSET,
                                        &data,
                                        0);

        // Unlock the Multi-function register
        temp_result = aDIO_General_Internal_LockMultiFunction(Handle, FALSE, 0);
        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = temp_result;

    } while(0);

    return result;
}

/*
    Retrieves the current Compare value.
*/
aDIO_Error aDIO_Port_Internal_GetCompare(   aDIO_Handle Handle,
                                            uint8*      Compare)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result;
    uint32 data;

    do
    {
        // Lock the Multi-function register to Compare mode
        result = aDIO_General_Internal_LockMultiFunction(
                                Handle,
                                TRUE,
                                ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_COMPARE);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Read the Multi-function register to get compare value
        result = aDIO_General_Internal_RegOp(
                                        Handle,
                                        ADIO_INTRFC_REG_OP_READ,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_SIZE,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_BLK,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_OFFSET,
                                        &data,
                                        0);

        // Copy out the Compare value
        if(result == ADIO_ERROR_CODE_SUCCESS)
            *Compare = (uint8)data;

        // Unlock the Multi-function register
        temp_result = aDIO_General_Internal_LockMultiFunction(Handle, FALSE, 0);
        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = temp_result;

    } while(0);

    return result;
}

/*
    Sets the desired Compare value.
*/
aDIO_Error aDIO_Port_Internal_SetCompare(   aDIO_Handle Handle,
                                            uint8       Compare)
{
    aDIO_Error result = ADIO_ERROR_CODE_SUCCESS;
    aDIO_Error temp_result;
    uint32 data;

    do
    {
        // Lock the Multi-function register to Compare mode
        result = aDIO_General_Internal_LockMultiFunction(
                                Handle,
                                TRUE,
                                ADIO_GENERAL_INTERNAL_MULTIFUNC_MODE_COMPARE);
        if(result != ADIO_ERROR_CODE_SUCCESS)
            break;

        // Write the Multi-function register to set compare value
        data = Compare;
        result = aDIO_General_Internal_RegOp(
                                        Handle,
                                        ADIO_INTRFC_REG_OP_WRITE,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_SIZE,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_BLK,
                                        ADIO_REGMAP_INTERNAL_MULTIFUNC_OFFSET,
                                        &data,
                                        0);

        // Unlock the Multi-function register
        temp_result = aDIO_General_Internal_LockMultiFunction(Handle, FALSE, 0);
        if(result == ADIO_ERROR_CODE_SUCCESS)
            result = temp_result;

    } while(0);

    return result;
}

//============================================================================
// End of Port Internal Functions
//============================================================================

//============================================================================
// End of Port Internal Module
//============================================================================

//============================================================================
// End of Internal Module
//============================================================================

