/**
    @file

    @brief
        This file defines internal functions for the 'Port' module of the
        aDIOLib.

    @verbatim
    --------------------------------------------------------------------------
    This file and its contents are copyright (C) RTD Embedded Technologies,
    Inc.  All Rights Reserved.

    This software is licensed as described in the RTD End-User Software License
    Agreement.  For a copy of this agreement, refer to the file LICENSE.TXT
    (which should be included with this software) or contact RTD Embedded
    Technologies, Inc.
    --------------------------------------------------------------------------
    @endverbatim

    $Id: aDIOLib_Port_Internal.h 55052 2011-08-16 19:33:01Z jstrouse $
*/

#ifndef __ADIOLIB_PORT_INTERNAL_H__
#define __ADIOLIB_PORT_INTERNAL_H__

#include "aDIOLib_Port.h"

#ifdef __cplusplus
extern "C" {
#endif


//============================================================================
//! @defgroup aDIO_Internal aDIOLib Internal Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Port_Internal aDIOLib Port Internal Module
//! @{
//============================================================================

//============================================================================
//! @defgroup aDIO_Port_Internal_Func aDIOLib Port Internal Functions
//! @{
//============================================================================

/**
    @brief
        Gets the current 'direction' (input or output) of Port 0.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Port0

        A pointer to a uint8 variable which will be filled with the current
        direction of port 0. If a bit's value is zero it means the corresponding
        bit of port 0 is an input. Similarly, a bit with a value of one means
        the corresponding bit of port 0 is an output.

*/
aDIO_Error aDIO_Port_Internal_GetP0Direction(   aDIO_Handle Handle,
                                                uint8*      Port0);

/**
    @brief
        Gets the current 'direction' (input or output) of Port 1.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Port1

        A pointer to a aDIO_Port_Dir_P1 variable which will be filled with the
        current direction of port 1.

*/
aDIO_Error aDIO_Port_Internal_GetP1Direction(   aDIO_Handle         Handle,
                                                aDIO_Port_Dir_P1*   Port1);

/**
    @brief
        Sets the current 'direction' (input or output) of Port 0.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Port0

        A uint8 value describing what direction to set each bit. If a bit's
        value is zero the corresponding bit of port 0 will be an input.
        Similarly, a bit with a value of one will make the corresponding bit of
        port 0 an output. (This value can be 'built' by ORing ADIO_PORT_BIT_*
        values together.)

*/
aDIO_Error aDIO_Port_Internal_SetP0Direction(   aDIO_Handle Handle,
                                                uint8       Port0);

/**
    @brief
        Sets the current 'direction' (input or output) of Port 1.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Port1

        The direction to set port 1 to.

*/
aDIO_Error aDIO_Port_Internal_SetP1Direction(   aDIO_Handle         Handle,
                                                aDIO_Port_Dir_P1    Port1);

/**
    @brief
        Retrieves the current Mask value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Mask

        A pointer to a uint8 value which is filled with the current digital IO
        mask.

*/
aDIO_Error aDIO_Port_Internal_GetMask(  aDIO_Handle Handle,
                                        uint8*      Mask);

/**
    @brief
        Sets the desired Mask value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Mask

        A uint8 value describing the digital IO mask to set. (This value can be
        'built' by ORing together ADIO_PORT_BIT_* values.

*/
aDIO_Error aDIO_Port_Internal_SetMask(  aDIO_Handle Handle,
                                        uint8       Mask);

/**
    @brief
        Retrieves the current Compare value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Compare

        A pointer to a uint8 value which is filled with the current digital IO
        compare value.

*/
aDIO_Error aDIO_Port_Internal_GetCompare(   aDIO_Handle Handle,
                                            uint8*      Compare);

/**
    @brief
        Sets the desired Compare value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Compare

        A uint8 value to set the digital IO compare value to.

*/
aDIO_Error aDIO_Port_Internal_SetCompare(   aDIO_Handle Handle,
                                            uint8       Compare);

//============================================================================
//! @} End of Port Internal Functions
//============================================================================

//============================================================================
//! @defgroup aDIO_Port_Internal_Macro aDIOLib Port Internal Macros
//! @{
//============================================================================

/**
    @brief
        Checks the validity of an aDIO_Port_DIO_Port value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Port

        The value to check.

    @warning
        If the value is not valid, an appropriate error message will be logged
        and the function using this macro will immediately return
        ADIO_ERROR_CODE_BAD_PARAM.

*/
#define ADIO_PORT_INTERNAL_CHECK_PORT(Handle, Port) \
    switch(Port)\
    {\
    case ADIO_PORT_DIO_PORT_0:\
    case ADIO_PORT_DIO_PORT_1:\
        break;\
    default:\
        return ADIO_LOG_INTERNAL_LOG_BAD_PARAM_INT(\
                                                Handle,\
                                                ADIO_ERROR_CATEGORY_PORT,\
                                                Port);\
    }

/**
    @brief
        Checks the validity of an aDIO_Port_Dir_P1 value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Dir

        The value to check.

    @warning
        If the value is not valid, an appropriate error message will be logged
        and the function using this macro will immediately return
        ADIO_ERROR_CODE_BAD_PARAM.

*/
#define ADIO_PORT_INTERNAL_CHECK_PORT1_DIR(Handle, Dir) \
    switch(Dir)\
    {\
    case ADIO_PORT_DIR_P1_INPUT:\
    case ADIO_PORT_DIR_P1_OUTPUT:\
        break;\
    default:\
        return ADIO_LOG_INTERNAL_LOG_BAD_PARAM_INT(\
                                                Handle,\
                                                ADIO_ERROR_CATEGORY_PORT,\
                                                Dir);\
    }

/**
    @brief
        Checks the validity of an aDIO_Port_Strobe value.

    @param
        Handle

        A valid handle to the aDIO device to be controlled.

    @param
        Strobe

        The value to check.

    @warning
        If the value is not valid, an appropriate error message will be logged
        and the function using this macro will immediately return
        ADIO_ERROR_CODE_BAD_PARAM.

*/
#define ADIO_PORT_INTERNAL_CHECK_STROBE(Handle, Strobe) \
    switch(Strobe)\
    {\
    case ADIO_PORT_STROBE_0:\
    case ADIO_PORT_STROBE_1:\
        break;\
    default:\
        return ADIO_LOG_INTERNAL_LOG_BAD_PARAM_INT(\
                                                Handle,\
                                                ADIO_ERROR_CATEGORY_PORT,\
                                                Strobe);\
    }

//============================================================================
//! @} End of Port Internal Macros
//============================================================================

//============================================================================
//! @} End of Port Internal Module
//============================================================================

//============================================================================
//! @} End of Internal Module
//============================================================================


#ifdef __cplusplus
}
#endif

#endif // __ADIOLIB_PORT_INTERNAL_H__
